@@ -48,6 +48,8 @@ enum CommandIds
     InspectionCommand,
     RemoveFromDiskCommand,
     FlushCommand,
+    SecretCommand,
+    UpgradeCommand,
     CustomCommand = 0xffff
 };
 
