@@ -119,7 +119,7 @@ class Filter : public FilterBase {
 public:
     typedef QSharedPointer<Filter> Ptr;
 
-    QHash<QByteArray, Sink::QueryBase::Comparator> propertyFilter;
+    QHash<QByteArrayList, Sink::QueryBase::Comparator> propertyFilter;
 
     Filter(FilterBase::Ptr source, DataStoreQuery *store)
         : FilterBase(source, store)
@@ -158,8 +158,21 @@ public:
 
     bool matchesFilter(const ApplicationDomain::ApplicationDomainType &entity) {
         for (const auto &filterProperty : propertyFilter.keys()) {
-            const auto property = entity.getProperty(filterProperty);
+            QVariant property;
+            if (filterProperty.size() == 1) {
+                property = entity.getProperty(filterProperty[0]);
+            } else {
+                QVariantList propList;
+                for (const auto &propName : filterProperty) {
+                    propList.push_back(entity.getProperty(propName));
+                }
+                property = propList;
+            }
             const auto comparator = propertyFilter.value(filterProperty);
+            //We can't deal with a fulltext filter
+            if (comparator.comparator == QueryBase::Comparator::Fulltext) {
+                continue;
+            }
             if (!comparator.matches(property)) {
                 SinkTraceCtx(mDatastore->mLogCtx) << "Filtering entity due to property mismatch on filter: " << entity.identifier() << "Property: " << filterProperty << property << " Filter:" << comparator.value;
                 return false;
@@ -208,6 +221,7 @@ public:
     };
 
     QSet<QByteArray> mReducedValues;
+    QSet<QByteArray> mIncrementallyReducedValues;
     QHash<QByteArray, QByteArray> mSelectedValues;
     QByteArray mReductionProperty;
     QByteArray mSelectionProperty;
@@ -225,6 +239,12 @@ public:
 
     virtual ~Reduce(){}
 
+    void updateComplete() Q_DECL_OVERRIDE
+    {
+        SinkTraceCtx(mDatastore->mLogCtx) << "Reduction update is complete.";
+        mIncrementallyReducedValues.clear();
+    }
+
     static QByteArray getByteArray(const QVariant &value) {
         if (value.type() == QVariant::DateTime) {
             return value.toDateTime().toString().toLatin1();
@@ -242,22 +262,29 @@ public:
         return false;
     }
 
-    QByteArray reduceOnValue(const QVariant &reductionValue, QMap<QByteArray, QVariant> &aggregateValues)
+    struct ReductionResult {
+        QByteArray selection;
+        QVector<QByteArray> aggregateIds;
+        QMap<QByteArray, QVariant> aggregateValues;
+    };
+
+    ReductionResult reduceOnValue(const QVariant &reductionValue)
     {
+        QMap<QByteArray, QVariant> aggregateValues;
         QVariant selectionResultValue;
         QByteArray selectionResult;
-        auto results = indexLookup(mReductionProperty, reductionValue);
+        const auto results = indexLookup(mReductionProperty, reductionValue);
         for (auto &aggregator : mAggregators) {
             aggregator.reset();
         }
-
+        QVector<QByteArray> reducedAndFilteredResults;
         for (const auto &r : results) {
             readEntity(r, [&, this](const Sink::ApplicationDomain::ApplicationDomainType &entity, Sink::Operation operation) {
                 //We need to apply all property filters that we have until the reduction, because the index lookup was unfiltered.
                 if (!matchesFilter(entity)) {
                     return;
                 }
-
+                reducedAndFilteredResults << r;
                 Q_ASSERT(operation != Sink::Operation_Removal);
                 for (auto &aggregator : mAggregators) {
                     if (!aggregator.property.isEmpty()) {
@@ -277,59 +304,90 @@ public:
         for (auto &aggregator : mAggregators) {
             aggregateValues.insert(aggregator.resultProperty, aggregator.result());
         }
-        return selectionResult;
+        return {selectionResult, reducedAndFilteredResults, aggregateValues};
     }
 
     bool next(const std::function<void(const ResultSet::Result &)> &callback) Q_DECL_OVERRIDE {
         bool foundValue = false;
         while(!foundValue && mSource->next([this, callback, &foundValue](const ResultSet::Result &result) {
-                if (result.operation == Sink::Operation_Removal) {
-                    callback(result);
-                    return false;
+                const auto reductionValue = [&] {
+                    const auto v = result.entity.getProperty(mReductionProperty);
+                    //Because we also get Operation_Removal for filtered entities. We use the fact that actually removed entites
+                    //won't have the property to reduce on.
+                    //TODO: Perhaps find a cleaner solutoin than abusing Operation::Removed for filtered properties.
+                    if (v.isNull() && result.operation == Sink::Operation_Removal) {
+                        //For removals we have to read the last revision to get a value, and thus be able to find the correct thread.
+                        QVariant reductionValue;
+                        readPrevious(result.entity.identifier(), [&] (const ApplicationDomain::ApplicationDomainType &prev) {
+                            Q_ASSERT(result.entity.identifier() == prev.identifier());
+                            reductionValue = prev.getProperty(mReductionProperty);
+                        });
+                        return reductionValue;
+                    } else {
+                        return v;
+                    }
+                }();
+                if (reductionValue.isNull()) {
+                    SinkTraceCtx(mDatastore->mLogCtx) << "No reduction value: " << result.entity.identifier();
+                    //We failed to find a value to reduce on, so ignore this entity.
+                    //Can happen if the entity was already removed and we have no previous revision.
+                    return;
                 }
-                auto reductionValue = result.entity.getProperty(mReductionProperty);
-                const auto &reductionValueBa = getByteArray(reductionValue);
+                const auto reductionValueBa = getByteArray(reductionValue);
                 if (!mReducedValues.contains(reductionValueBa)) {
                     //Only reduce every value once.
                     mReducedValues.insert(reductionValueBa);
-                    QMap<QByteArray, QVariant> aggregateValues;
-                    auto selectionResult = reduceOnValue(reductionValue, aggregateValues);
+                    SinkTraceCtx(mDatastore->mLogCtx) << "Reducing new value: " << result.entity.identifier() << reductionValueBa;
+                    auto reductionResult = reduceOnValue(reductionValue);
 
-                    mSelectedValues.insert(reductionValueBa, selectionResult);
-                    readEntity(selectionResult, [&, this](const Sink::ApplicationDomain::ApplicationDomainType &entity, Sink::Operation operation) {
-                        callback({entity, operation, aggregateValues});
+                    //This can happen if we get a removal message from a filtered entity and all entites of the reduction are filtered.
+                    if (reductionResult.selection.isEmpty()) {
+                        return;
+                    }
+                    mSelectedValues.insert(reductionValueBa, reductionResult.selection);
+                    readEntity(reductionResult.selection, [&](const Sink::ApplicationDomain::ApplicationDomainType &entity, Sink::Operation operation) {
+                        callback({entity, operation, reductionResult.aggregateValues, reductionResult.aggregateIds});
                         foundValue = true;
                     });
                 } else {
                     //During initial query, do nothing. The lookup above will take care of it.
                     //During updates adjust the reduction according to the modification/addition or removal
-                    if (mIncremental) {
-                        //redo the reduction
-                        QMap<QByteArray, QVariant> aggregateValues;
-                        auto selectionResult = reduceOnValue(reductionValue, aggregateValues);
+                    //We have to redo the reduction for every element, because of the aggregation values.
+                    if (mIncremental && !mIncrementallyReducedValues.contains(reductionValueBa)) {
+                        SinkTraceCtx(mDatastore->mLogCtx) << "Incremental reduction update: " << result.entity.identifier() << reductionValueBa;
+                        mIncrementallyReducedValues.insert(reductionValueBa);
+                        //Redo the reduction to find new aggregated values
+                        auto selectionResult = reduceOnValue(reductionValue);
 
-                        //TODO if old and new are the same a modification would be enough
                         auto oldSelectionResult = mSelectedValues.take(reductionValueBa);
-                        if (oldSelectionResult == selectionResult) {
-                            mSelectedValues.insert(reductionValueBa, selectionResult);
-                            readEntity(selectionResult, [&, this](const Sink::ApplicationDomain::ApplicationDomainType &entity, Sink::Operation) {
-                                callback({entity, Sink::Operation_Modification, aggregateValues});
+                        SinkTraceCtx(mDatastore->mLogCtx) << "Old selection result: " << oldSelectionResult << " New selection result: " << selectionResult.selection;
+                        if (oldSelectionResult == selectionResult.selection) {
+                            mSelectedValues.insert(reductionValueBa, selectionResult.selection);
+                            Q_ASSERT(!selectionResult.selection.isEmpty());
+                            readEntity(selectionResult.selection, [&](const Sink::ApplicationDomain::ApplicationDomainType &entity, Sink::Operation) {
+                                callback({entity, Sink::Operation_Modification, selectionResult.aggregateValues, selectionResult.aggregateIds});
                             });
                         } else {
                             //remove old result
-                            readEntity(oldSelectionResult, [&, this](const Sink::ApplicationDomain::ApplicationDomainType &entity, Sink::Operation) {
-                                callback({entity, Sink::Operation_Removal});
-                            });
+                            //If mSelectedValues did not containthe value, oldSelectionResult will be empty.(Happens if entites have been filtered)
+                            if (!oldSelectionResult.isEmpty()) {
+                                readEntity(oldSelectionResult, [&](const Sink::ApplicationDomain::ApplicationDomainType &entity, Sink::Operation) {
+                                    callback({entity, Sink::Operation_Removal});
+                                });
+                            }
 
-                            //add new result
-                            mSelectedValues.insert(reductionValueBa, selectionResult);
-                            readEntity(selectionResult, [&, this](const Sink::ApplicationDomain::ApplicationDomainType &entity, Sink::Operation) {
-                                callback({entity, Sink::Operation_Creation, aggregateValues});
-                            });
+                            //If the last item has been removed, then there's nothing to add
+                            if (!selectionResult.selection.isEmpty()) {
+                                //add new result
+                                mSelectedValues.insert(reductionValueBa, selectionResult.selection);
+                                Q_ASSERT(!selectionResult.selection.isEmpty());
+                                readEntity(selectionResult.selection, [&](const Sink::ApplicationDomain::ApplicationDomainType &entity, Sink::Operation) {
+                                    callback({entity, Sink::Operation_Creation, selectionResult.aggregateValues, selectionResult.aggregateIds});
+                                });
+                            }
                         }
                     }
                 }
-                return false;
             }))
         {}
         return foundValue;
@@ -370,7 +428,7 @@ public:
                 }))
             {}
             mBloomed = true;
-            propertyFilter.insert(mBloomProperty, mBloomValue);
+            propertyFilter.insert({mBloomProperty}, mBloomValue);
             return foundValue;
         } else {
             //Filter on bloom value
@@ -422,6 +480,11 @@ void DataStoreQuery::readEntity(const QByteArray &key, const BufferCallback &res
     mStore.readLatest(mType, key, resultCallback);
 }
 
+void DataStoreQuery::readPrevious(const QByteArray &key, const std::function<void (const ApplicationDomain::ApplicationDomainType &)> &callback)
+{
+    mStore.readPrevious(mType, key, mStore.maxRevision(), callback);
+}
+
 QVector<QByteArray> DataStoreQuery::indexLookup(const QByteArray &property, const QVariant &value)
 {
     return mStore.indexLookup(mType, property, value);
@@ -524,6 +587,7 @@ void DataStoreQuery::setupQuery(const Sink::QueryBase &query_)
 {
     auto query = query_;
     auto baseFilters = query.getBaseFilters();
+    //Resolve any subqueries we have
     for (const auto &k : baseFilters.keys()) {
         const auto comparator = baseFilters.value(k);
         if (comparator.value.canConvert<Query>()) {
@@ -534,26 +598,26 @@ void DataStoreQuery::setupQuery(const Sink::QueryBase &query_)
     }
     query.setBaseFilters(baseFilters);
 
-    FilterBase::Ptr baseSet;
-    QSet<QByteArray> remainingFilters = query.getBaseFilters().keys().toSet();
     QByteArray appliedSorting;
-    if (!query.ids().isEmpty()) {
-        mSource = Source::Ptr::create(query.ids().toVector(), this);
-        baseSet = mSource;
-    } else {
-        QSet<QByteArray> appliedFilters;
 
-        auto resultSet = mStore.indexLookup(mType, query, appliedFilters, appliedSorting);
-        remainingFilters = remainingFilters - appliedFilters;
-
-        // We do a full scan if there were no indexes available to create the initial set.
-        if (appliedFilters.isEmpty()) {
-            mSource = Source::Ptr::create(mStore.fullScan(mType), this);
+    //Determine initial set
+    mSource = [&]() {
+        if (!query.ids().isEmpty()) {
+            //We have a set of ids as a starting point
+            return Source::Ptr::create(query.ids().toVector(), this);
         } else {
-            mSource = Source::Ptr::create(resultSet, this);
+            QSet<QByteArrayList> appliedFilters;
+            auto resultSet = mStore.indexLookup(mType, query, appliedFilters, appliedSorting);
+            if (!appliedFilters.isEmpty()) {
+                //We have an index lookup as starting point
+                return Source::Ptr::create(resultSet, this);
+            }
+            // We do a full scan if there were no indexes available to create the initial set (this is going to be expensive for large sets).
+            return Source::Ptr::create(mStore.fullScan(mType), this);
         }
-        baseSet = mSource;
-    }
+    }();
+
+    FilterBase::Ptr baseSet = mSource;
     if (!query.getBaseFilters().isEmpty()) {
         auto filter = Filter::Ptr::create(baseSet, this);
         //For incremental queries the remaining filters are not sufficient
@@ -599,7 +663,7 @@ QVector<QByteArray> DataStoreQuery::loadIncrementalResultSet(qint64 baseRevision
 
 ResultSet DataStoreQuery::update(qint64 baseRevision)
 {
-    SinkTraceCtx(mLogCtx) << "Executing query update from revision " << baseRevision;
+    SinkTraceCtx(mLogCtx) << "Executing query update from revision " << baseRevision << " to revision " << mStore.maxRevision();
     auto incrementalResultSet = loadIncrementalResultSet(baseRevision);
     SinkTraceCtx(mLogCtx) << "Incremental changes: " << incrementalResultSet;
     mSource->add(incrementalResultSet);
@@ -619,17 +683,23 @@ ResultSet DataStoreQuery::update(qint64 baseRevision)
 void DataStoreQuery::updateComplete()
 {
     mSource->mIncrementalIds.clear();
+    auto source = mCollector;
+    while (source) {
+        source->updateComplete();
+        source = source->mSource;
+    }
 }
 
 ResultSet DataStoreQuery::execute()
 {
     SinkTraceCtx(mLogCtx) << "Executing query";
 
+    Q_ASSERT(mCollector);
     ResultSet::ValueGenerator generator = [this](const ResultSet::Callback &callback) -> bool {
         if (mCollector->next([this, callback](const ResultSet::Result &result) {
                 if (result.operation != Sink::Operation_Removal) {
                     SinkTraceCtx(mLogCtx) << "Got initial result: " << result.entity.identifier() << result.operation;
-                    callback(ResultSet::Result{result.entity, Sink::Operation_Creation, result.aggregateValues});
+                    callback(ResultSet::Result{result.entity, Sink::Operation_Creation, result.aggregateValues, result.aggregateIds});
                 }
             }))
         {
