@@ -45,15 +45,6 @@ QVariant parseString<Sink::ApplicationDomain::Reference>(const QString &s)
     return QVariant::fromValue(Sink::ApplicationDomain::Reference{s.toLatin1()});
 }
 
-template <>
-QVariant parseString<Sink::ApplicationDomain::BLOB>(const QString &s)
-{
-    //TODO copy path
-    // return QVariant::fromValue(Sink::ApplicationDomain::BLOB{s.toLatin1()});
-    Q_ASSERT(false);
-    return QVariant{};
-}
-
 template <>
 QVariant parseString<bool>(const QString &s)
 {
@@ -83,10 +74,16 @@ QVariant parseString<QList<QByteArray>>(const QString &s)
     return QVariant::fromValue(result);
 }
 
+template <>
+QVariant parseString<QStringList>(const QString &s)
+{
+    return s.split(',');
+}
+
 template <>
 QVariant parseString<QDateTime>(const QString &s)
 {
-    return QVariant::fromValue(QDateTime::fromString(s));
+    return QVariant::fromValue(QDateTime::fromString(s, Qt::ISODate));
 }
 
 template <>
