@@ -27,6 +27,7 @@
 #include "entitybuffer.h"
 #include "entity_generated.h"
 #include "mail/threadindexer.h"
+#include "mail/fulltextindexer.h"
 #include "domainadaptor.h"
 #include "typeimplementations_p.h"
 
@@ -37,7 +38,7 @@ using namespace Sink::ApplicationDomain;
     MAPPER.addMapping<ENTITYTYPE::PROPERTY, Sink::ApplicationDomain::Buffer::ENTITYTYPE, Sink::ApplicationDomain::Buffer::ENTITYTYPE##Builder>(&Sink::ApplicationDomain::Buffer::ENTITYTYPE::LOWERCASEPROPERTY, &Sink::ApplicationDomain::Buffer::ENTITYTYPE##Builder::add_##LOWERCASEPROPERTY);
 
 typedef IndexConfig<Mail,
-        ValueIndex<Mail::Date>,
+        SortedIndex<Mail::Date>,
         ValueIndex<Mail::Folder>,
         ValueIndex<Mail::ParentMessageId>,
         ValueIndex<Mail::MessageId>,
@@ -45,7 +46,8 @@ typedef IndexConfig<Mail,
         SortedIndex<Mail::Folder, Mail::Date>,
         SecondaryIndex<Mail::MessageId, Mail::ThreadId>,
         SecondaryIndex<Mail::ThreadId, Mail::MessageId>,
-        CustomSecondaryIndex<Mail::MessageId, Mail::ThreadId, ThreadIndexer>
+        CustomSecondaryIndex<Mail::MessageId, Mail::ThreadId, ThreadIndexer>,
+        CustomSecondaryIndex<Mail::Subject, Mail::Subject, FulltextIndexer>
     > MailIndexConfig;
 
 typedef IndexConfig<Folder,
@@ -62,9 +64,20 @@ typedef IndexConfig<Addressbook,
     > AddressbookIndexConfig;
 
 typedef IndexConfig<Event,
-        ValueIndex<Event::Uid>
+        ValueIndex<Event::Uid>,
+        SortedIndex<Event::StartTime>,
+        SampledPeriodIndex<Event::StartTime, Event::EndTime>
     > EventIndexConfig;
 
+typedef IndexConfig<Todo,
+        ValueIndex<Todo::Uid>
+    > TodoIndexConfig;
+
+typedef IndexConfig<Calendar,
+        ValueIndex<Calendar::Name>
+    > CalendarIndexConfig;
+
+
 
 void TypeImplementation<Mail>::configure(TypeIndex &index)
 {
@@ -199,7 +212,11 @@ void TypeImplementation<Event>::configure(PropertyMapper &propertyMapper)
     SINK_REGISTER_SERIALIZER(propertyMapper, Event, Summary, summary);
     SINK_REGISTER_SERIALIZER(propertyMapper, Event, Description, description);
     SINK_REGISTER_SERIALIZER(propertyMapper, Event, Uid, uid);
-    SINK_REGISTER_SERIALIZER(propertyMapper, Event, Attachment, attachment);
+    SINK_REGISTER_SERIALIZER(propertyMapper, Event, StartTime, startTime);
+    SINK_REGISTER_SERIALIZER(propertyMapper, Event, EndTime, endTime);
+    SINK_REGISTER_SERIALIZER(propertyMapper, Event, AllDay, allDay);
+    SINK_REGISTER_SERIALIZER(propertyMapper, Event, Ical, ical);
+    SINK_REGISTER_SERIALIZER(propertyMapper, Event, Calendar, calendar);
 }
 
 void TypeImplementation<Event>::configure(IndexPropertyMapper &)
@@ -207,3 +224,51 @@ void TypeImplementation<Event>::configure(IndexPropertyMapper &)
 
 }
 
+
+void TypeImplementation<Todo>::configure(TypeIndex &index)
+{
+    TodoIndexConfig::configure(index);
+}
+
+QMap<QByteArray, int> TypeImplementation<Todo>::typeDatabases()
+{
+    return merge(QMap<QByteArray, int>{{QByteArray{Todo::name} + ".main", 0}}, TodoIndexConfig::databases());
+}
+
+void TypeImplementation<Todo>::configure(PropertyMapper &propertyMapper)
+{
+    SINK_REGISTER_SERIALIZER(propertyMapper, Todo, Uid, uid);
+    SINK_REGISTER_SERIALIZER(propertyMapper, Todo, Summary, summary);
+    SINK_REGISTER_SERIALIZER(propertyMapper, Todo, Description, description);
+    SINK_REGISTER_SERIALIZER(propertyMapper, Todo, CompletedDate, completedDate);
+    SINK_REGISTER_SERIALIZER(propertyMapper, Todo, DueDate, dueDate);
+    SINK_REGISTER_SERIALIZER(propertyMapper, Todo, StartDate, startDate);
+    SINK_REGISTER_SERIALIZER(propertyMapper, Todo, Status, status);
+    SINK_REGISTER_SERIALIZER(propertyMapper, Todo, Priority, priority);
+    SINK_REGISTER_SERIALIZER(propertyMapper, Todo, Categories, categories);
+    SINK_REGISTER_SERIALIZER(propertyMapper, Todo, Ical, ical);
+    SINK_REGISTER_SERIALIZER(propertyMapper, Todo, Calendar, calendar);
+}
+
+void TypeImplementation<Todo>::configure(IndexPropertyMapper &)
+{
+
+}
+
+
+void TypeImplementation<Calendar>::configure(TypeIndex &index)
+{
+    CalendarIndexConfig::configure(index);
+}
+
+QMap<QByteArray, int> TypeImplementation<Calendar>::typeDatabases()
+{
+    return merge(QMap<QByteArray, int>{{QByteArray{Calendar::name} + ".main", 0}}, CalendarIndexConfig::databases());
+}
+
+void TypeImplementation<Calendar>::configure(PropertyMapper &propertyMapper)
+{
+    SINK_REGISTER_SERIALIZER(propertyMapper, Calendar, Name, name);
+}
+
+void TypeImplementation<Calendar>::configure(IndexPropertyMapper &) {}
