@@ -18,11 +18,14 @@
  */
 #pragma once
 
+#include "sink_export.h"
 #include "applicationdomaintype.h"
 
 #include "mail_generated.h"
 #include "folder_generated.h"
 #include "event_generated.h"
+#include "todo_generated.h"
+#include "calendar_generated.h"
 #include "contact_generated.h"
 #include "addressbook_generated.h"
 
@@ -40,7 +43,7 @@ namespace Sink {
 namespace ApplicationDomain {
 
 template<>
-class TypeImplementation<Sink::ApplicationDomain::Mail> {
+class SINK_EXPORT TypeImplementation<Sink::ApplicationDomain::Mail> {
 public:
     typedef Sink::ApplicationDomain::Buffer::Mail Buffer;
     typedef Sink::ApplicationDomain::Buffer::MailBuilder BufferBuilder;
@@ -51,7 +54,7 @@ public:
 };
 
 template<>
-class TypeImplementation<Sink::ApplicationDomain::Folder> {
+class SINK_EXPORT TypeImplementation<Sink::ApplicationDomain::Folder> {
 public:
     typedef Sink::ApplicationDomain::Buffer::Folder Buffer;
     typedef Sink::ApplicationDomain::Buffer::FolderBuilder BufferBuilder;
@@ -62,7 +65,7 @@ public:
 };
 
 template<>
-class TypeImplementation<Sink::ApplicationDomain::Contact> {
+class SINK_EXPORT TypeImplementation<Sink::ApplicationDomain::Contact> {
 public:
     typedef Sink::ApplicationDomain::Buffer::Contact Buffer;
     typedef Sink::ApplicationDomain::Buffer::ContactBuilder BufferBuilder;
@@ -73,7 +76,7 @@ public:
 };
 
 template<>
-class TypeImplementation<Sink::ApplicationDomain::Addressbook> {
+class SINK_EXPORT TypeImplementation<Sink::ApplicationDomain::Addressbook> {
 public:
     typedef Sink::ApplicationDomain::Buffer::Addressbook Buffer;
     typedef Sink::ApplicationDomain::Buffer::AddressbookBuilder BufferBuilder;
@@ -84,7 +87,7 @@ public:
 };
 
 template<>
-class TypeImplementation<Sink::ApplicationDomain::Event> {
+class SINK_EXPORT TypeImplementation<Sink::ApplicationDomain::Event> {
 public:
     typedef Sink::ApplicationDomain::Buffer::Event Buffer;
     typedef Sink::ApplicationDomain::Buffer::EventBuilder BufferBuilder;
@@ -94,5 +97,27 @@ public:
     static QMap<QByteArray, int> typeDatabases();
 };
 
+template<>
+class SINK_EXPORT TypeImplementation<Sink::ApplicationDomain::Todo> {
+public:
+    typedef Sink::ApplicationDomain::Buffer::Todo Buffer;
+    typedef Sink::ApplicationDomain::Buffer::TodoBuilder BufferBuilder;
+    static void configure(TypeIndex &);
+    static void configure(PropertyMapper &);
+    static void configure(IndexPropertyMapper &indexPropertyMapper);
+    static QMap<QByteArray, int> typeDatabases();
+};
+
+template<>
+class SINK_EXPORT TypeImplementation<Sink::ApplicationDomain::Calendar> {
+public:
+    typedef Sink::ApplicationDomain::Buffer::Calendar Buffer;
+    typedef Sink::ApplicationDomain::Buffer::CalendarBuilder BufferBuilder;
+    static void configure(TypeIndex &);
+    static void configure(PropertyMapper &);
+    static void configure(IndexPropertyMapper &indexPropertyMapper);
+    static QMap<QByteArray, int> typeDatabases();
+};
+
 }
 }
