@@ -21,6 +21,7 @@
 
 #include "pipeline.h"
 #include "synchronizer.h"
+#include "inspector.h"
 #include "commandprocessor.h"
 #include "definitions.h"
 #include "storage.h"
@@ -45,6 +46,49 @@ GenericResource::~GenericResource()
 {
 }
 
+void GenericResource::setSecret(const QString &s)
+{
+    if (mSynchronizer) {
+        mSynchronizer->setSecret(s);
+    }
+    if (mInspector) {
+        mInspector->setSecret(s);
+    }
+}
+
+bool GenericResource::checkForUpgrade()
+{
+    auto store = Sink::Storage::DataStore(Sink::storageLocation(), mResourceContext.instanceId(), Sink::Storage::DataStore::ReadOnly);
+    //We rely on the store already having been created in the pipeline constructor before this get's called.
+    Q_ASSERT(store.exists());
+    const auto currentDatabaseVersion = Storage::DataStore::databaseVersion(store.createTransaction(Storage::DataStore::ReadOnly));
+    if (currentDatabaseVersion != Sink::latestDatabaseVersion()) {
+        SinkLog() << "Starting database upgrade from " << currentDatabaseVersion << " to " << Sink::latestDatabaseVersion();
+
+        bool nukeDatabases = false;
+        //Only apply the necessary updates.
+        for (int i = currentDatabaseVersion; i < Sink::latestDatabaseVersion(); i++) {
+            //TODO implement specific upgrade paths where applicable, and only nuke otherwise
+            nukeDatabases = true;
+        }
+        if (nukeDatabases) {
+            SinkLog() << "Wiping all databases during upgrade, you will have to resync.";
+            //Right now upgrading just means removing all local storage so we will resync
+            Sink::Storage::DataStore(Sink::storageLocation(), mResourceContext.instanceId(), Sink::Storage::DataStore::ReadWrite).removeFromDisk();
+            Sink::Storage::DataStore(Sink::storageLocation(), mResourceContext.instanceId() + ".userqueue", Sink::Storage::DataStore::ReadWrite).removeFromDisk();
+            Sink::Storage::DataStore(Sink::storageLocation(), mResourceContext.instanceId() + ".synchronizerqueue", Sink::Storage::DataStore::ReadWrite).removeFromDisk();
+            Sink::Storage::DataStore(Sink::storageLocation(), mResourceContext.instanceId() + ".changereplay", Sink::Storage::DataStore::ReadWrite).removeFromDisk();
+            Sink::Storage::DataStore(Sink::storageLocation(), mResourceContext.instanceId() + ".synchronization", Sink::Storage::DataStore::ReadWrite).removeFromDisk();
+        }
+        auto store = Sink::Storage::DataStore(Sink::storageLocation(), mResourceContext.instanceId(), Sink::Storage::DataStore::ReadWrite);
+        auto t = store.createTransaction(Storage::DataStore::ReadWrite);
+        Storage::DataStore::setDatabaseVersion(t, Sink::latestDatabaseVersion());
+        SinkLog() << "Finished database upgrade to " << Sink::latestDatabaseVersion();
+        return true;
+    }
+    return false;
+}
+
 void GenericResource::setupPreprocessors(const QByteArray &type, const QVector<Sink::Preprocessor *> &preprocessors)
 {
     mPipeline->setPreprocessors(type, preprocessors);
@@ -61,6 +105,7 @@ void GenericResource::setupSynchronizer(const QSharedPointer<Synchronizer> &sync
 
 void GenericResource::setupInspector(const QSharedPointer<Inspector> &inspector)
 {
+    mInspector = inspector;
     mProcessor->setInspector(inspector);
 }
 
@@ -79,6 +124,7 @@ qint64 GenericResource::diskUsage(const QByteArray &instanceIdentifier)
     size += Sink::Storage::DataStore(Sink::storageLocation(), instanceIdentifier + ".userqueue", Sink::Storage::DataStore::ReadOnly).diskUsage();
     size += Sink::Storage::DataStore(Sink::storageLocation(), instanceIdentifier + ".synchronizerqueue", Sink::Storage::DataStore::ReadOnly).diskUsage();
     size += Sink::Storage::DataStore(Sink::storageLocation(), instanceIdentifier + ".changereplay", Sink::Storage::DataStore::ReadOnly).diskUsage();
+    size += Sink::Storage::DataStore(Sink::storageLocation(), instanceIdentifier + ".synchronization", Sink::Storage::DataStore::ReadOnly).diskUsage();
     return size;
 }
 
@@ -120,7 +166,3 @@ void GenericResource::setLowerBoundRevision(qint64 revision)
     updateLowerBoundRevision();
 }
 
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wundefined-reinterpret-cast"
-#include "genericresource.moc"
-#pragma clang diagnostic pop
