@@ -37,7 +37,7 @@ class SINK_EXPORT GenericResource : public Resource
 {
 public:
     GenericResource(const Sink::ResourceContext &context, const QSharedPointer<Pipeline> &pipeline = QSharedPointer<Pipeline>());
-    virtual ~GenericResource();
+    virtual ~GenericResource() Q_DECL_OVERRIDE;
 
     virtual void processCommand(int commandId, const QByteArray &data) Q_DECL_OVERRIDE;
     virtual void setLowerBoundRevision(qint64 revision) Q_DECL_OVERRIDE;
@@ -47,6 +47,9 @@ public:
     static void removeFromDisk(const QByteArray &instanceIdentifier);
     static qint64 diskUsage(const QByteArray &instanceIdentifier);
 
+    virtual void setSecret(const QString &s) Q_DECL_OVERRIDE;
+    virtual bool checkForUpgrade() Q_DECL_OVERRIDE;
+
     //TODO Remove this API, it's only used in tests
     KAsync::Job<void> synchronizeWithSource(const Sink::QueryBase &query);
     //TODO Remove this API, it's only used in tests
@@ -68,6 +71,7 @@ private:
     QSharedPointer<Pipeline> mPipeline;
     QSharedPointer<CommandProcessor> mProcessor;
     QSharedPointer<Synchronizer> mSynchronizer;
+    QSharedPointer<Inspector> mInspector;
     int mError;
     qint64 mClientLowerBoundRevision;
 };
