@@ -29,7 +29,9 @@ public:
         int code;
     };
 
+    Index(const QString &storageRoot, const QString &dbName, const QString &indexName, Sink::Storage::DataStore::AccessMode mode = Sink::Storage::DataStore::ReadOnly);
     Index(const QString &storageRoot, const QString &name, Sink::Storage::DataStore::AccessMode mode = Sink::Storage::DataStore::ReadOnly);
+    Index(const QString &storageRoot, const Sink::Storage::DbLayout &layout, Sink::Storage::DataStore::AccessMode mode = Sink::Storage::DataStore::ReadOnly);
     Index(const QByteArray &name, Sink::Storage::DataStore::Transaction &);
 
     void add(const QByteArray &key, const QByteArray &value);
@@ -39,6 +41,10 @@ public:
         bool matchSubStringKeys = false);
     QByteArray lookup(const QByteArray &key);
 
+    void rangeLookup(const QByteArray &lowerBound, const QByteArray &upperBound,
+        const std::function<void(const QByteArray &value)> &resultHandler,
+        const std::function<void(const Error &error)> &errorHandler);
+
 private:
     Q_DISABLE_COPY(Index);
     Sink::Storage::DataStore::Transaction mTransaction;
