@@ -33,10 +33,12 @@
 #include "common/revisionupdate_generated.h"
 #include "common/notification_generated.h"
 #include "common/revisionreplayed_generated.h"
+#include "common/secret_generated.h"
 
 #include <QLocalServer>
 #include <QLocalSocket>
 #include <QTimer>
+#include <chrono>
 
 Listener::Listener(const QByteArray &resourceInstanceIdentifier, const QByteArray &resourceType, QObject *parent)
     : QObject(parent),
@@ -64,14 +66,14 @@ Listener::Listener(const QByteArray &resourceInstanceIdentifier, const QByteArra
 
     m_checkConnectionsTimer = std::unique_ptr<QTimer>(new QTimer);
     m_checkConnectionsTimer->setSingleShot(true);
-    m_checkConnectionsTimer->setInterval(1000);
     connect(m_checkConnectionsTimer.get(), &QTimer::timeout, [this]() {
         if (m_connections.isEmpty()) {
             SinkTrace() << QString("No connections, shutting down.");
             quit();
         }
     });
-    m_checkConnectionsTimer->start();
+    //Give plenty of time during the first start.
+    m_checkConnectionsTimer->start(std::chrono::milliseconds{60000});
 
     // TODO: experiment with different timeouts
     //      or even just drop down to invoking the method queued? => invoke queued unless we need throttling
@@ -86,6 +88,14 @@ Listener::~Listener()
     closeAllConnections();
 }
 
+void Listener::checkForUpgrade()
+{
+    if (loadResource().checkForUpgrade()) {
+        //Close the resource to ensure no transactions are open
+        m_resource.reset(nullptr);
+    }
+}
+
 void Listener::emergencyAbortAllConnections()
 {
     Sink::Notification n;
@@ -99,10 +109,10 @@ void Listener::emergencyAbortAllConnections()
             SinkWarning() << "Sending panic";
             client.socket->write("PANIC");
             client.socket->waitForBytesWritten();
-            disconnect(client.socket, 0, this, 0);
+            disconnect(client.socket, nullptr, this, nullptr);
             client.socket->abort();
             delete client.socket;
-            client.socket = 0;
+            client.socket = nullptr;
         }
     }
 
@@ -113,10 +123,11 @@ void Listener::closeAllConnections()
 {
     for (Client &client : m_connections) {
         if (client.socket) {
-            disconnect(client.socket, 0, this, 0);
+            disconnect(client.socket, nullptr, this, nullptr);
+            client.socket->flush();
             client.socket->close();
             delete client.socket;
-            client.socket = 0;
+            client.socket = nullptr;
         }
     }
 
@@ -179,7 +190,7 @@ void Listener::checkConnections()
     if (m_connections.isEmpty()) {
         loadResource().setLowerBoundRevision(std::numeric_limits<qint64>::max());
     }
-    m_checkConnectionsTimer->start();
+    m_checkConnectionsTimer->start(std::chrono::milliseconds{1000});
 }
 
 void Listener::onDataAvailable()
@@ -240,6 +251,17 @@ void Listener::processCommand(int commandId, uint messageId, const QByteArray &c
             }
             break;
         }
+        case Sink::Commands::SecretCommand: {
+            flatbuffers::Verifier verifier((const uint8_t *)commandBuffer.constData(), commandBuffer.size());
+            if (Sink::Commands::VerifySecretBuffer(verifier)) {
+                auto buffer = Sink::Commands::GetSecret(commandBuffer.constData());
+                loadResource().setSecret(QString{buffer->secret()->c_str()});
+            } else {
+                SinkWarning() << "received invalid command";
+            }
+            break;
+        }
+
         case Sink::Commands::SynchronizeCommand:
         case Sink::Commands::InspectionCommand:
         case Sink::Commands::DeleteEntityCommand:
@@ -277,6 +299,9 @@ void Listener::processCommand(int commandId, uint messageId, const QByteArray &c
             }
             m_exiting = true;
         } break;
+        case Sink::Commands::UpgradeCommand:
+            //Because we synchronously run the update directly on resource start, we know that the upgrade is complete once this message completes.
+            break;
         default:
             if (commandId > Sink::Commands::CustomCommand) {
                 SinkLog() << QString("Received custom command from %1: ").arg(client.name) << commandId;
@@ -339,9 +364,9 @@ bool Listener::processClientBuffer(Client &client)
         return false;
     }
 
-    const uint messageId = *(uint *)client.commandBuffer.constData();
-    const int commandId = *(int *)(client.commandBuffer.constData() + sizeof(uint));
-    const uint size = *(uint *)(client.commandBuffer.constData() + sizeof(int) + sizeof(uint));
+    const uint messageId = *(const uint *)client.commandBuffer.constData();
+    const int commandId = *(const int *)(client.commandBuffer.constData() + sizeof(uint));
+    const uint size = *(const uint *)(client.commandBuffer.constData() + sizeof(int) + sizeof(uint));
     SinkTrace() << "Received message. Id:" << messageId << " CommandId: " << commandId << " Size: " << size;
 
     // TODO: reject messages above a certain size?
