@@ -23,7 +23,6 @@
 
 #include <QByteArray>
 #include <QVector>
-#include <QUuid>
 #include <QDebug>
 #include <QTime>
 #include "entity_generated.h"
@@ -64,8 +63,7 @@ public:
 Pipeline::Pipeline(const ResourceContext &context, const Sink::Log::Context &ctx) : QObject(nullptr), d(new Private(context, ctx))
 {
     //Create main store immediately on first start
-    d->entityStore.startTransaction(DataStore::ReadWrite);
-    d->entityStore.commitTransaction();
+    d->entityStore.initialize();
 }
 
 Pipeline::~Pipeline()
@@ -135,7 +133,7 @@ KAsync::Job<qint64> Pipeline::newEntity(void const *command, size_t size)
     QByteArray key;
     if (createEntity->entityId()) {
         key = QByteArray(reinterpret_cast<char const *>(createEntity->entityId()->Data()), createEntity->entityId()->size());
-        if (d->entityStore.contains(bufferType, key)) {
+        if (!key.isEmpty() && d->entityStore.contains(bufferType, key)) {
             SinkErrorCtx(d->logCtx) << "An entity with this id already exists: " << key;
             return KAsync::error<qint64>(0);
         }
@@ -162,7 +160,7 @@ KAsync::Job<qint64> Pipeline::newEntity(void const *command, size_t size)
 
     auto adaptorFactory = Sink::AdaptorFactoryRegistry::instance().getFactory(d->resourceContext.resourceType, bufferType);
     if (!adaptorFactory) {
-        SinkWarningCtx(d->logCtx) << "no adaptor factory for type " << bufferType;
+        SinkWarningCtx(d->logCtx) << "no adaptor factory for type " << bufferType << d->resourceContext.resourceType;
         return KAsync::error<qint64>(0);
     }
 
@@ -260,7 +258,21 @@ KAsync::Job<qint64> Pipeline::modifiedEntity(void const *command, size_t size)
         return KAsync::error<qint64>(0);
     }
 
-    auto newEntity = d->entityStore.applyDiff(bufferType, current, diff, deletions);
+    //We avoid overwriting local changes that haven't been played back yet with remote modifications
+    QSet<QByteArray> excludeProperties;
+    if (!replayToSource) { //We assume this means the change is coming from the source already
+        d->entityStore.readRevisions(bufferType, diff.identifier(), baseRevision, [&] (const QByteArray &uid, qint64 revision, const Sink::EntityBuffer &entity) {
+            if (entity.metadataBuffer()) {
+                if (auto metadata = GetMetadata(entity.metadataBuffer())) {
+                    if (metadata->operation() == Operation_Modification && metadata->modifiedProperties()) {
+                        excludeProperties += BufferUtils::fromVector(*metadata->modifiedProperties()).toSet();
+                    }
+                }
+            }
+        });
+    }
+
+    auto newEntity = d->entityStore.applyDiff(bufferType, current, diff, deletions, excludeProperties);
 
     bool isMove = false;
     if (modifyEntity->targetResource()) {
@@ -295,9 +307,11 @@ KAsync::Job<qint64> Pipeline::modifiedEntity(void const *command, size_t size)
 
     //The entity is either being copied or moved
     if (newEntity.resourceInstanceIdentifier() != d->resourceContext.resourceInstanceIdentifier) {
-        SinkTraceCtx(d->logCtx) << "Moving entity to new resource " << newEntity.identifier() << newEntity.resourceInstanceIdentifier();
-        newEntity.setChangedProperties(newEntity.availableProperties().toSet());
-        return create(bufferType, newEntity)
+        auto copy = *ApplicationDomain::ApplicationDomainType::getInMemoryCopy<ApplicationDomain::ApplicationDomainType>(newEntity, newEntity.availableProperties());
+        copy.setResource(newEntity.resourceInstanceIdentifier());
+        copy.setChangedProperties(copy.availableProperties().toSet());
+        SinkTraceCtx(d->logCtx) << "Moving entity to new resource " << copy.identifier() << copy.resourceInstanceIdentifier();
+        return create(bufferType, copy)
             .then([=](const KAsync::Error &error) {
                 if (!error) {
                     SinkTraceCtx(d->logCtx) << "Move of " << current.identifier() << "was successfull";
