@@ -32,9 +32,11 @@
 #include "common/revisionreplayed_generated.h"
 #include "common/inspection_generated.h"
 #include "common/flush_generated.h"
+#include "common/secret_generated.h"
 #include "common/entitybuffer.h"
 #include "common/bufferutils.h"
 #include "common/test.h"
+#include "common/secretstore.h"
 #include "log.h"
 
 #include <QCoreApplication>
@@ -44,8 +46,9 @@
 #include <QDataStream>
 #include <QBuffer>
 #include <QTime>
+#include <QStandardPaths>
 
-static void queuedInvoke(const std::function<void()> &f, QObject *context = 0)
+static void queuedInvoke(const std::function<void()> &f, QObject *context = nullptr)
 {
     auto timer = QSharedPointer<QTimer>::create();
     timer->setSingleShot(true);
@@ -80,6 +83,7 @@ class ResourceAccess::Private
 {
 public:
     Private(const QByteArray &name, const QByteArray &instanceIdentifier, ResourceAccess *ra);
+    ~Private();
     KAsync::Job<void> tryToConnect();
     KAsync::Job<void> initializeSocket();
     void abortPendingOperations();
@@ -103,6 +107,9 @@ ResourceAccess::Private::Private(const QByteArray &name, const QByteArray &insta
     : resourceName(name), resourceInstanceIdentifier(instanceIdentifier), messageId(0), openingSocket(false)
 {
 }
+ResourceAccess::Private::~Private()
+{
+}
 
 void ResourceAccess::Private::abortPendingOperations()
 {
@@ -123,8 +130,9 @@ void ResourceAccess::Private::abortPendingOperations()
 
 void ResourceAccess::Private::callCallbacks()
 {
-    for (auto id : completeCommands.keys()) {
-        const bool success = completeCommands.value(id);
+    const auto commandIds = completeCommands.keys();
+    for (auto id : commandIds) {
+        const bool success = completeCommands.take(id);
         // We remove the callbacks first because the handler can kill resourceaccess directly
         const auto callbacks = resultHandler.values(id);
         resultHandler.remove(id);
@@ -141,9 +149,8 @@ void ResourceAccess::Private::callCallbacks()
 // Connects to server and returns connected socket on success
 KAsync::Job<QSharedPointer<QLocalSocket>> ResourceAccess::connectToServer(const QByteArray &identifier)
 {
-    auto s = QSharedPointer<QLocalSocket>::create();
+    auto s = QSharedPointer<QLocalSocket>{new QLocalSocket, &QObject::deleteLater};
     return KAsync::start<QSharedPointer<QLocalSocket>>([identifier, s](KAsync::Future<QSharedPointer<QLocalSocket>> &future) {
-        s->setServerName(identifier);
         auto context = new QObject;
         QObject::connect(s.data(), &QLocalSocket::connected, context, [&future, &s, context]() {
             Q_ASSERT(s);
@@ -151,11 +158,14 @@ KAsync::Job<QSharedPointer<QLocalSocket>> ResourceAccess::connectToServer(const
             future.setValue(s);
             future.setFinished();
         });
-        QObject::connect(s.data(), static_cast<void (QLocalSocket::*)(QLocalSocket::LocalSocketError)>(&QLocalSocket::error), context, [&future, context](QLocalSocket::LocalSocketError) {
+        QObject::connect(s.data(), static_cast<void (QLocalSocket::*)(QLocalSocket::LocalSocketError)>(&QLocalSocket::error), context, [&future, &s, context](QLocalSocket::LocalSocketError localSocketError) {
+            const auto errorString = s->errorString();
+            const auto name = s->fullServerName();
             delete context;
-            future.setError(-1, "Failed to connect to server.");
+            future.setError(localSocketError, QString("Failed to connect to socket %1: %2").arg(name).arg(errorString));
+            future.setError({1, QString("Failed to connect to socket %1: %2 %3").arg(name).arg(localSocketError).arg(errorString)});
         });
-        s->open();
+        s->connectToServer(identifier);
     });
 }
 
@@ -173,9 +183,9 @@ KAsync::Job<void> ResourceAccess::Private::tryToConnect()
                             static int waitTime = 10;
                             static int timeout = 20000;
                             static int maxRetries = timeout / waitTime;
-                            if (*counter > maxRetries) {
-                                SinkTrace() << "Giving up";
-                                return KAsync::error<KAsync::ControlFlowFlag>("Failed to connect to socket");
+                            if (*counter >= maxRetries) {
+                                SinkTrace() << "Giving up after " << *counter << "tries";
+                                return KAsync::error<KAsync::ControlFlowFlag>(error);
                             } else {
                                 *counter = *counter + 1;
                                 return KAsync::wait(waitTime).then(KAsync::value(KAsync::Continue));
@@ -208,16 +218,27 @@ KAsync::Job<void> ResourceAccess::Private::initializeSocket()
                             return KAsync::error();
                         }
                         args << resourceInstanceIdentifier << resourceName;
+
+                        //Prefer a binary next to this binary, otherwise fall-back to PATH. Necessary for MacOS bundles because the bundle is not in the PATH.
+                        auto executable = QStandardPaths::findExecutable("sink_synchronizer", {QCoreApplication::applicationDirPath()});
+                        if (executable.isEmpty()) {
+                            executable = QStandardPaths::findExecutable("sink_synchronizer");
+                        }
+                        if (executable.isEmpty()) {
+                            SinkError() << "Failed to find the sink_synchronizer binary in the paths: " << QCoreApplication::applicationDirPath();
+                            return KAsync::error("Failed to find the sink_synchronizer binary.");
+                        }
                         qint64 pid = 0;
-                        if (QProcess::startDetached("sink_synchronizer", args, QDir::homePath(), &pid)) {
+                        SinkLog() << "Starting resource " << executable << args.join(" ") << "Home path: " << QDir::homePath();
+                        if (QProcess::startDetached(executable, args, QDir::homePath(), &pid)) {
                             SinkTrace() << "Started resource " << pid;
                             return tryToConnect()
                                 .onError([this, args](const KAsync::Error &error) {
-                                    SinkWarning() << "Failed to connect to started resource: sink_synchronizer " << args;
+                                    SinkError() << "Failed to connect to started resource: sink_synchronizer " << args;
                                 });
                         } else {
-                            SinkWarning() << "Failed to start resource";
-                            return KAsync::error(-1, "Failed to start resource");
+                            SinkError() << "Failed to start resource";
+                            return KAsync::error("Failed to start resource.");
                         }
                     } else {
                         SinkTrace() << "Connected to resource, without having to start it.";
@@ -232,8 +253,14 @@ KAsync::Job<void> ResourceAccess::Private::initializeSocket()
 ResourceAccess::ResourceAccess(const QByteArray &resourceInstanceIdentifier, const QByteArray &resourceType)
     : ResourceAccessInterface(), d(new Private(resourceType, resourceInstanceIdentifier, this))
 {
-    mResourceStatus = Sink::ApplicationDomain::OfflineStatus;
+    mResourceStatus = Sink::ApplicationDomain::NoStatus;
     SinkTrace() << "Starting access";
+
+    QObject::connect(&SecretStore::instance(), &SecretStore::secretAvailable, this, [this] (const QByteArray &resourceId) {
+        if (resourceId == d->resourceInstanceIdentifier) {
+            sendSecret(SecretStore::instance().resourceSecret(d->resourceInstanceIdentifier)).exec();
+        }
+    });
 }
 
 ResourceAccess::~ResourceAccess()
@@ -242,6 +269,7 @@ ResourceAccess::~ResourceAccess()
     if (!d->resultHandler.isEmpty()) {
         SinkWarning() << "Left jobs running while shutting down ResourceAccess: " << d->resultHandler.keys();
     }
+    delete d;
 }
 
 QByteArray ResourceAccess::resourceName() const
@@ -387,6 +415,15 @@ KAsync::Job<void> ResourceAccess::sendFlushCommand(int flushType, const QByteArr
     return sendCommand(Sink::Commands::FlushCommand, fbb);
 }
 
+KAsync::Job<void> ResourceAccess::sendSecret(const QString &secret)
+{
+    flatbuffers::FlatBufferBuilder fbb;
+    auto s = fbb.CreateString(secret.toStdString());
+    auto location = Sink::Commands::CreateSecret(fbb, s);
+    Sink::Commands::FinishSecretBuffer(fbb, location);
+    return sendCommand(Sink::Commands::SecretCommand, fbb);
+}
+
 void ResourceAccess::open()
 {
     if (d->socket && d->socket->isValid()) {
@@ -404,10 +441,11 @@ void ResourceAccess::open()
             [this, time](const KAsync::Error &error) {
                 d->openingSocket = false;
                 if (error) {
-                    SinkWarning() << "Failed to initialize socket " << error.errorMessage;
+                    SinkError() << "Failed to initialize socket " << error;
                     d->abortPendingOperations();
                 } else {
                     SinkTrace() << "Socket is initialized." << Log::TraceTime(time->elapsed());
+                    Q_ASSERT(d->socket);
                     QObject::connect(d->socket.data(), &QLocalSocket::disconnected, this, &ResourceAccess::disconnected);
                     QObject::connect(d->socket.data(), SIGNAL(error(QLocalSocket::LocalSocketError)), this, SLOT(connectionError(QLocalSocket::LocalSocketError)));
                     QObject::connect(d->socket.data(), &QIODevice::readyRead, this, &ResourceAccess::readResourceMessage);
@@ -485,8 +523,10 @@ void ResourceAccess::connected()
 
     // Reenqueue pending commands, we failed to send them
     processPendingCommandQueue();
-    // Send queued commands
-    processCommandQueue();
+    auto secret = SecretStore::instance().resourceSecret(d->resourceInstanceIdentifier);
+    if (!secret.isEmpty()) {
+        sendSecret(secret).exec();
+    }
 
     emit ready(true);
 }
@@ -494,6 +534,9 @@ void ResourceAccess::connected()
 void ResourceAccess::disconnected()
 {
     SinkLog() << QString("Disconnected from %1").arg(d->socket->fullServerName());
+    //Ensure we read all remaining data before closing the socket.
+    //This is required on windows at least.
+    readResourceMessage();
     d->socket->close();
     emit ready(false);
 }
@@ -507,6 +550,10 @@ void ResourceAccess::connectionError(QLocalSocket::LocalSocketError error)
         Sink::Notification n;
         n.type = Sink::Notification::Status;
         emit notification(n);
+        Sink::Notification crashNotification;
+        crashNotification.type = Sink::Notification::Error;
+        crashNotification.code = Sink::ApplicationDomain::ResourceCrashedError;
+        emit notification(crashNotification);
         d->abortPendingOperations();
     } else if (error == QLocalSocket::PeerClosedError) {
         SinkLog() << "The resource closed the connection.";
@@ -522,15 +569,17 @@ void ResourceAccess::connectionError(QLocalSocket::LocalSocketError error)
 
 void ResourceAccess::readResourceMessage()
 {
-    if (!d->socket || !d->socket->isValid()) {
+    if (!d->socket) {
         SinkWarning() << "No socket available";
         return;
     }
 
-    d->partialMessageBuffer += d->socket->readAll();
+    if (d->socket->bytesAvailable()) {
+        d->partialMessageBuffer += d->socket->readAll();
 
-    // should be scheduled rather than processed all at once
-    while (processMessageBuffer()) {
+        // should be scheduled rather than processed all at once
+        while (processMessageBuffer()) {
+        }
     }
 }
 
@@ -557,16 +606,19 @@ bool ResourceAccess::processMessageBuffer()
 {
     static const int headerSize = Commands::headerSize();
     if (d->partialMessageBuffer.size() < headerSize) {
-        SinkWarning() << "command too small";
+        //This is not an error
+        SinkTrace() << "command too small, smaller than headerSize: " << d->partialMessageBuffer.size() << headerSize;
         return false;
     }
 
     // const uint messageId = *(int*)(d->partialMessageBuffer.constData());
-    const int commandId = *(int *)(d->partialMessageBuffer.constData() + sizeof(uint));
-    const uint size = *(int *)(d->partialMessageBuffer.constData() + sizeof(int) + sizeof(uint));
+    const int commandId = *(const int *)(d->partialMessageBuffer.constData() + sizeof(uint));
+    const uint size = *(const int *)(d->partialMessageBuffer.constData() + sizeof(int) + sizeof(uint));
 
-    if (size > (uint)(d->partialMessageBuffer.size() - headerSize)) {
-        SinkWarning() << "command too small";
+    const uint availableMessageSize = d->partialMessageBuffer.size() - headerSize;
+    if (size > availableMessageSize) {
+        //This is not an error
+        SinkTrace() << "command too small, message smaller than advertised: " << availableMessageSize << headerSize;
         return false;
     }
 
@@ -638,9 +690,16 @@ bool ResourceAccess::processMessageBuffer()
     return d->partialMessageBuffer.size() >= headerSize;
 }
 
+
+
+ResourceAccessFactory::ResourceAccessFactory()
+{
+
+}
+
 ResourceAccessFactory &ResourceAccessFactory::instance()
 {
-    static ResourceAccessFactory *instance = 0;
+    static ResourceAccessFactory *instance = nullptr;
     if (!instance) {
         instance = new ResourceAccessFactory;
     }
@@ -652,17 +711,21 @@ Sink::ResourceAccess::Ptr ResourceAccessFactory::getAccess(const QByteArray &ins
     if (!mCache.contains(instanceIdentifier)) {
         // Reuse the pointer if something else kept the resourceaccess alive
         if (mWeakCache.contains(instanceIdentifier)) {
-            auto sharedPointer = mWeakCache.value(instanceIdentifier).toStrongRef();
-            if (sharedPointer) {
+            if (auto sharedPointer = mWeakCache.value(instanceIdentifier).toStrongRef()) {
                 mCache.insert(instanceIdentifier, sharedPointer);
             }
         }
         if (!mCache.contains(instanceIdentifier)) {
             // Create a new instance if necessary
-            auto sharedPointer = Sink::ResourceAccess::Ptr::create(instanceIdentifier, resourceType);
+            auto sharedPointer = Sink::ResourceAccess::Ptr{new Sink::ResourceAccess(instanceIdentifier, resourceType), &QObject::deleteLater};
             QObject::connect(sharedPointer.data(), &Sink::ResourceAccess::ready, sharedPointer.data(), [this, instanceIdentifier](bool ready) {
                 if (!ready) {
-                    mCache.remove(instanceIdentifier);
+                    //We want to remove, but we don't want shared pointer to be destroyed until end of the function as this might trigger further steps.
+                    auto ptr = mCache.take(instanceIdentifier);
+                    if (auto timer = mTimer.take(instanceIdentifier)) {
+                        timer->stop();
+                    }
+                    Q_UNUSED(ptr);
                 }
             });
             mCache.insert(instanceIdentifier, sharedPointer);
@@ -670,15 +733,18 @@ Sink::ResourceAccess::Ptr ResourceAccessFactory::getAccess(const QByteArray &ins
         }
     }
     if (!mTimer.contains(instanceIdentifier)) {
-        auto timer = new QTimer;
+        auto timer = QSharedPointer<QTimer>::create();
         timer->setSingleShot(true);
         // Drop connection after 3 seconds (which is a random value)
-        QObject::connect(timer, &QTimer::timeout, timer, [this, instanceIdentifier]() { mCache.remove(instanceIdentifier); });
+        QObject::connect(timer.data(), &QTimer::timeout, timer.data(), [this, instanceIdentifier]() {
+            //We want to remove, but we don't want shared pointer to be destroyed until end of the function as this might trigger further steps.
+            auto ptr = mCache.take(instanceIdentifier);
+            Q_UNUSED(ptr);
+        });
         timer->setInterval(3000);
         mTimer.insert(instanceIdentifier, timer);
     }
-    auto timer = mTimer.value(instanceIdentifier);
-    timer->start();
+    mTimer.value(instanceIdentifier)->start();
     return mCache.value(instanceIdentifier);
 }
 }
