@@ -44,6 +44,7 @@ public:
     typedef QSharedPointer<ResourceAccessInterface> Ptr;
 
     ResourceAccessInterface()
+        : QObject()
     {
     }
     virtual ~ResourceAccessInterface()
@@ -80,6 +81,11 @@ public:
         return KAsync::null<void>();
     }
 
+    virtual KAsync::Job<void> sendSecret(const QString &secret)
+    {
+        return KAsync::null<void>();
+    }
+
     int getResourceStatus() const
     {
         return mResourceStatus;
@@ -105,7 +111,7 @@ public:
     typedef QSharedPointer<ResourceAccess> Ptr;
 
     ResourceAccess(const QByteArray &resourceInstanceIdentifier, const QByteArray &resourceType);
-    ~ResourceAccess();
+    virtual ~ResourceAccess() Q_DECL_OVERRIDE;
 
     QByteArray resourceName() const;
     bool isReady() const;
@@ -121,6 +127,7 @@ public:
     KAsync::Job<void>
     sendInspectionCommand(int inspectionType,const QByteArray &inspectionId, const QByteArray &domainType, const QByteArray &entityId, const QByteArray &property, const QVariant &expecedValue) Q_DECL_OVERRIDE;
     KAsync::Job<void> sendFlushCommand(int flushType, const QByteArray &flushId) Q_DECL_OVERRIDE;
+    KAsync::Job<void> sendSecret(const QString &secret) Q_DECL_OVERRIDE;
     /**
      * Tries to connect to server, and returns a connected socket on success.
      */
@@ -161,8 +168,11 @@ public:
     static ResourceAccessFactory &instance();
     Sink::ResourceAccess::Ptr getAccess(const QByteArray &instanceIdentifier, const QByteArray resourceType);
 
+private:
+    ResourceAccessFactory();
+
     QHash<QByteArray, QWeakPointer<Sink::ResourceAccess>> mWeakCache;
     QHash<QByteArray, Sink::ResourceAccess::Ptr> mCache;
-    QHash<QByteArray, QTimer *> mTimer;
+    QHash<QByteArray, QSharedPointer<QTimer>> mTimer;
 };
 }
