@@ -85,7 +85,9 @@ static Sink::Storage::DbLayout dbLayout(const QByteArray &instanceId)
         mergeImpl(map, ApplicationDomain::TypeImplementation<ApplicationDomain::Folder>::typeDatabases());
         mergeImpl(map, ApplicationDomain::TypeImplementation<ApplicationDomain::Contact>::typeDatabases());
         mergeImpl(map, ApplicationDomain::TypeImplementation<ApplicationDomain::Addressbook>::typeDatabases());
+        mergeImpl(map, ApplicationDomain::TypeImplementation<ApplicationDomain::Calendar>::typeDatabases());
         mergeImpl(map, ApplicationDomain::TypeImplementation<ApplicationDomain::Event>::typeDatabases());
+        mergeImpl(map, ApplicationDomain::TypeImplementation<ApplicationDomain::Todo>::typeDatabases());
         return merge(baseDbs(), map);
     }();
     return {instanceId, databases};
@@ -96,14 +98,6 @@ class EntityStore::Private {
 public:
     Private(const ResourceContext &context, const Sink::Log::Context &ctx) : resourceContext(context), logCtx(ctx.subContext("entitystore"))
     {
-        static bool initialized = false;
-        if (!initialized) {
-            if (QDir{}.mkpath(entityBlobStorageDir())) {
-                initialized = true;
-            } else {
-                SinkWarningCtx(logCtx) << "Failed to create the directory: " << entityBlobStorageDir();
-            }
-        }
     }
 
     ResourceContext resourceContext;
@@ -113,7 +107,7 @@ public:
 
     bool exists()
     {
-        return Sink::Storage::DataStore(Sink::storageLocation(), resourceContext.instanceId(), DataStore::ReadOnly).exists();
+        return DataStore(Sink::storageLocation(), resourceContext.instanceId(), DataStore::ReadOnly).exists();
     }
 
     DataStore::Transaction &getTransaction()
@@ -122,7 +116,7 @@ public:
             return transaction;
         }
 
-        Sink::Storage::DataStore store(Sink::storageLocation(), dbLayout(resourceContext.instanceId()), DataStore::ReadOnly);
+        DataStore store(Sink::storageLocation(), dbLayout(resourceContext.instanceId()), DataStore::ReadOnly);
         transaction = store.createTransaction(DataStore::ReadOnly);
         return transaction;
     }
@@ -153,22 +147,11 @@ public:
         return index;
     }
 
-    ApplicationDomain::ApplicationDomainType createApplicationDomainType(const QByteArray &type, const QByteArray &uid, qint64 revision, const EntityBuffer &buffer)
+    ApplicationDomainType createApplicationDomainType(const QByteArray &type, const QByteArray &uid, qint64 revision, const EntityBuffer &buffer)
     {
         auto adaptor = resourceContext.adaptorFactory(type).createAdaptor(buffer.entity(), &typeIndex(type));
-        return ApplicationDomain::ApplicationDomainType{resourceContext.instanceId(), uid, revision, adaptor};
+        return ApplicationDomainType{resourceContext.instanceId(), uid, revision, adaptor};
     }
-
-    QString entityBlobStorageDir()
-    {
-        return Sink::resourceStorageLocation(resourceContext.instanceId()) + "/blob";
-    }
-
-    QString entityBlobStoragePath(const QByteArray &id)
-    {
-        return entityBlobStorageDir() +"/"+ id;
-    }
-
 };
 
 EntityStore::EntityStore(const ResourceContext &context, const Log::Context &ctx)
@@ -177,27 +160,50 @@ EntityStore::EntityStore(const ResourceContext &context, const Log::Context &ctx
 
 }
 
-void EntityStore::startTransaction(Sink::Storage::DataStore::AccessMode accessMode)
+void EntityStore::initialize()
+{
+    //This function is only called in the resource code where we want to be able to write to the databse.
+
+    //Check for the existience of the db without creating it or the envrionment.
+    //This is required to be able to set the database version only in the case where we create a new database.
+    if (!Storage::DataStore::exists(Sink::storageLocation(), d->resourceContext.instanceId())) {
+        //The first time we open the environment we always want it to be read/write. Otherwise subsequent tries to open a write transaction will fail.
+        startTransaction(DataStore::ReadWrite);
+        //Create the database with the correct version if it wasn't existing before
+        SinkLogCtx(d->logCtx) << "Creating resource database.";
+        Storage::DataStore::setDatabaseVersion(d->transaction, Sink::latestDatabaseVersion());
+    } else {
+        //The first time we open the environment we always want it to be read/write. Otherwise subsequent tries to open a write transaction will fail.
+        startTransaction(DataStore::ReadWrite);
+    }
+    commitTransaction();
+}
+
+void EntityStore::startTransaction(DataStore::AccessMode accessMode)
 {
     SinkTraceCtx(d->logCtx) << "Starting transaction: " << accessMode;
     Q_ASSERT(!d->transaction);
-    Sink::Storage::DataStore store(Sink::storageLocation(), dbLayout(d->resourceContext.instanceId()), accessMode);
-    d->transaction = store.createTransaction(accessMode);
+    d->transaction = DataStore(Sink::storageLocation(), dbLayout(d->resourceContext.instanceId()), accessMode).createTransaction(accessMode);
 }
 
 void EntityStore::commitTransaction()
 {
     SinkTraceCtx(d->logCtx) << "Committing transaction";
+
+    for (const auto &type : d->indexByType.keys()) {
+        d->typeIndex(type).commitTransaction();
+    }
+
     Q_ASSERT(d->transaction);
     d->transaction.commit();
-    d->transaction = Storage::DataStore::Transaction();
+    d->transaction = {};
 }
 
 void EntityStore::abortTransaction()
 {
     SinkTraceCtx(d->logCtx) << "Aborting transaction";
     d->transaction.abort();
-    d->transaction = Storage::DataStore::Transaction();
+    d->transaction = {};
 }
 
 bool EntityStore::hasTransaction() const
@@ -205,36 +211,7 @@ bool EntityStore::hasTransaction() const
     return d->transaction;
 }
 
-void EntityStore::copyBlobs(ApplicationDomain::ApplicationDomainType &entity, qint64 newRevision)
-{
-    const auto directory = d->entityBlobStoragePath(entity.identifier());
-
-    for (const auto &property : entity.changedProperties()) {
-        const auto value = entity.getProperty(property);
-        if (value.canConvert<ApplicationDomain::BLOB>()) {
-            const auto blob = value.value<ApplicationDomain::BLOB>();
-            //Any blob that is not part of the storage yet has to be moved there.
-            if (blob.isExternal) {
-                auto oldPath = blob.value;
-                auto filePath = directory + QString("_%1%2.blob").arg(QString::number(newRevision)).arg(QString::fromLatin1(property));
-                //In case we hit the same revision again due to a rollback.
-                QFile::remove(filePath);
-                QFile origFile(oldPath);
-                if (!origFile.open(QIODevice::ReadWrite)) {
-                    SinkWarningCtx(d->logCtx) << "Failed to open the original file with write rights: " << origFile.errorString();
-                }
-                if (!origFile.rename(filePath)) {
-                    SinkWarningCtx(d->logCtx) << "Failed to move the file from: " << oldPath << " to " << filePath << ". " << origFile.errorString();
-                }
-                origFile.close();
-                SinkTraceCtx(d->logCtx) << "Moved blob property: " << property << " from " << oldPath << "to" << filePath;
-                entity.setProperty(property, QVariant::fromValue(ApplicationDomain::BLOB{filePath}));
-            }
-        }
-    }
-}
-
-bool EntityStore::add(const QByteArray &type, ApplicationDomain::ApplicationDomainType entity, bool replayToSource)
+bool EntityStore::add(const QByteArray &type, ApplicationDomainType entity, bool replayToSource)
 {
     if (entity.identifier().isEmpty()) {
         SinkWarningCtx(d->logCtx) << "Can't write entity with an empty identifier";
@@ -243,13 +220,11 @@ bool EntityStore::add(const QByteArray &type, ApplicationDomain::ApplicationDoma
 
     SinkTraceCtx(d->logCtx) << "New entity " << entity;
 
-    d->typeIndex(type).add(entity.identifier(), entity, d->transaction);
+    d->typeIndex(type).add(entity.identifier(), entity, d->transaction, d->resourceContext.instanceId());
 
     //The maxRevision may have changed meanwhile if the entity created sub-entities
     const qint64 newRevision = maxRevision() + 1;
 
-    copyBlobs(entity, newRevision);
-
     // Add metadata buffer
     flatbuffers::FlatBufferBuilder metadataFbb;
     auto metadataBuilder = MetadataBuilder(metadataFbb);
@@ -267,36 +242,36 @@ bool EntityStore::add(const QByteArray &type, ApplicationDomain::ApplicationDoma
             [&](const DataStore::Error &error) { SinkWarningCtx(d->logCtx) << "Failed to write entity" << entity.identifier() << newRevision; });
     DataStore::setMaxRevision(d->transaction, newRevision);
     DataStore::recordRevision(d->transaction, newRevision, entity.identifier(), type);
-    DataStore::recordUid(d->transaction, entity.identifier());
+    DataStore::recordUid(d->transaction, entity.identifier(), type);
     SinkTraceCtx(d->logCtx) << "Wrote entity: " << entity.identifier() << type << newRevision;
     return true;
 }
 
-ApplicationDomain::ApplicationDomainType EntityStore::applyDiff(const QByteArray &type, const ApplicationDomain::ApplicationDomainType &current, const ApplicationDomain::ApplicationDomainType &diff, const QByteArrayList &deletions) const
+ApplicationDomain::ApplicationDomainType EntityStore::applyDiff(const QByteArray &type, const ApplicationDomainType &current, const ApplicationDomainType &diff, const QByteArrayList &deletions, const QSet<QByteArray> &excludeProperties) const
 {
-    auto newEntity = *ApplicationDomain::ApplicationDomainType::getInMemoryRepresentation<ApplicationDomain::ApplicationDomainType>(current, current.availableProperties());
-
-    SinkTraceCtx(d->logCtx) << "Modified entity: " << newEntity;
+    SinkTraceCtx(d->logCtx) << "Applying diff: " << current.availableProperties() << "Deletions: " << deletions << "Changeset: " << diff.changedProperties() << "Excluded: " << excludeProperties;
+    auto newEntity = *ApplicationDomainType::getInMemoryRepresentation<ApplicationDomainType>(current, current.availableProperties());
 
     // Apply diff
-    //SinkTrace() << "Applying changed properties: " << changeset;
     for (const auto &property : diff.changedProperties()) {
-        const auto value = diff.getProperty(property);
-        if (value.isValid()) {
-            //SinkTrace() << "Setting property: " << property;
-            newEntity.setProperty(property, value);
+        if (!excludeProperties.contains(property)) {
+            const auto value = diff.getProperty(property);
+            if (value.isValid()) {
+                newEntity.setProperty(property, value);
+            }
         }
     }
 
     // Remove deletions
     for (const auto &property : deletions) {
-        //SinkTrace() << "Removing property: " << property;
-        newEntity.setProperty(property, QVariant());
+        if (!excludeProperties.contains(property)) {
+            newEntity.setProperty(property, QVariant());
+        }
     }
     return newEntity;
 }
 
-bool EntityStore::modify(const QByteArray &type, const ApplicationDomain::ApplicationDomainType &diff, const QByteArrayList &deletions, bool replayToSource)
+bool EntityStore::modify(const QByteArray &type, const ApplicationDomainType &diff, const QByteArrayList &deletions, bool replayToSource)
 {
     const auto current = readLatest(type, diff.identifier());
     if (current.identifier().isEmpty()) {
@@ -308,17 +283,14 @@ bool EntityStore::modify(const QByteArray &type, const ApplicationDomain::Applic
     return modify(type, current, newEntity, replayToSource);
 }
 
-bool EntityStore::modify(const QByteArray &type, const ApplicationDomain::ApplicationDomainType &current, ApplicationDomain::ApplicationDomainType newEntity, bool replayToSource)
+bool EntityStore::modify(const QByteArray &type, const ApplicationDomainType &current, ApplicationDomainType newEntity, bool replayToSource)
 {
     SinkTraceCtx(d->logCtx) << "Modified entity: " << newEntity;
 
-    d->typeIndex(type).remove(current.identifier(), current, d->transaction);
-    d->typeIndex(type).add(newEntity.identifier(), newEntity, d->transaction);
+    d->typeIndex(type).modify(newEntity.identifier(), current, newEntity, d->transaction, d->resourceContext.instanceId());
 
     const qint64 newRevision = DataStore::maxRevision(d->transaction) + 1;
 
-    copyBlobs(newEntity, newRevision);
-
     // Add metadata buffer
     flatbuffers::FlatBufferBuilder metadataFbb;
     {
@@ -348,7 +320,7 @@ bool EntityStore::modify(const QByteArray &type, const ApplicationDomain::Applic
     return true;
 }
 
-bool EntityStore::remove(const QByteArray &type, const Sink::ApplicationDomain::ApplicationDomainType &current, bool replayToSource)
+bool EntityStore::remove(const QByteArray &type, const ApplicationDomainType &current, bool replayToSource)
 {
     const auto uid = current.identifier();
     if (!exists(type, uid)) {
@@ -356,7 +328,7 @@ bool EntityStore::remove(const QByteArray &type, const Sink::ApplicationDomain::
         return false;
     }
 
-    d->typeIndex(type).remove(current.identifier(), current, d->transaction);
+    d->typeIndex(type).remove(current.identifier(), current, d->transaction, d->resourceContext.instanceId());
 
     SinkTraceCtx(d->logCtx) << "Removed entity " << current;
 
@@ -379,7 +351,7 @@ bool EntityStore::remove(const QByteArray &type, const Sink::ApplicationDomain::
             [&](const DataStore::Error &error) { SinkWarningCtx(d->logCtx) << "Failed to write entity" << uid << newRevision; });
     DataStore::setMaxRevision(d->transaction, newRevision);
     DataStore::recordRevision(d->transaction, newRevision, uid, type);
-    DataStore::removeUid(d->transaction, uid);
+    DataStore::removeUid(d->transaction, uid, type);
     return true;
 }
 
@@ -408,13 +380,6 @@ void EntityStore::cleanupEntityRevisionsUntil(qint64 revision)
                         DataStore::removeRevision(d->transaction, rev);
                         DataStore::mainDatabase(d->transaction, bufferType).remove(key);
                     }
-                    if (isRemoval) {
-                        QDir dir{d->entityBlobStorageDir()};
-                        const auto infoList = dir.entryInfoList(QStringList{} << QString{uid + "*"});
-                        for (const auto &fileInfo : infoList) {
-                            QFile::remove(fileInfo.filePath());
-                        }
-                    }
                     //Don't cleanup more than specified
                     if (rev >= revision) {
                         return false;
@@ -429,9 +394,11 @@ void EntityStore::cleanupEntityRevisionsUntil(qint64 revision)
 
 bool EntityStore::cleanupRevisions(qint64 revision)
 {
+    Q_ASSERT(d->exists());
     bool implicitTransaction = false;
     if (!d->transaction) {
-        startTransaction(Sink::Storage::DataStore::ReadWrite);
+        startTransaction(DataStore::ReadWrite);
+        Q_ASSERT(d->transaction);
         implicitTransaction = true;
     }
     const auto lastCleanRevision = DataStore::cleanedUpRevision(d->transaction);
@@ -469,19 +436,19 @@ QVector<QByteArray> EntityStore::fullScan(const QByteArray &type)
                 keys << uid;
                 return true;
             },
-            [&](const DataStore::Error &error) { SinkWarningCtx(d->logCtx) << "Error during query: " << error.message; });
+            [&](const DataStore::Error &error) { SinkWarningCtx(d->logCtx) << "Error during fullScan query: " << error.message; });
 
     SinkTraceCtx(d->logCtx) << "Full scan retrieved " << keys.size() << " results.";
     return keys.toList().toVector();
 }
 
-QVector<QByteArray> EntityStore::indexLookup(const QByteArray &type, const QueryBase &query, QSet<QByteArray> &appliedFilters, QByteArray &appliedSorting)
+QVector<QByteArray> EntityStore::indexLookup(const QByteArray &type, const QueryBase &query, QSet<QByteArrayList> &appliedFilters, QByteArray &appliedSorting)
 {
     if (!d->exists()) {
         SinkTraceCtx(d->logCtx) << "Database is not existing: " << type;
         return QVector<QByteArray>();
     }
-    return d->typeIndex(type).query(query, appliedFilters, appliedSorting, d->getTransaction());
+    return d->typeIndex(type).query(query, appliedFilters, appliedSorting, d->getTransaction(), d->resourceContext.instanceId());
 }
 
 QVector<QByteArray> EntityStore::indexLookup(const QByteArray &type, const QByteArray &property, const QVariant &value)
@@ -514,16 +481,17 @@ void EntityStore::indexLookup(const QByteArray &type, const QByteArray &property
 
 void EntityStore::readLatest(const QByteArray &type, const QByteArray &uid, const std::function<void(const QByteArray &uid, const EntityBuffer &entity)> callback)
 {
+    Q_ASSERT(d);
+    Q_ASSERT(!uid.isEmpty());
     auto db = DataStore::mainDatabase(d->getTransaction(), type);
     db.findLatest(uid,
-        [=](const QByteArray &key, const QByteArray &value) -> bool {
+        [=](const QByteArray &key, const QByteArray &value) {
             callback(DataStore::uidFromKey(key), Sink::EntityBuffer(value.data(), value.size()));
-            return false;
         },
-        [&](const DataStore::Error &error) { SinkWarningCtx(d->logCtx) << "Error during query: " << error.message << uid; });
+        [&](const DataStore::Error &error) { SinkWarningCtx(d->logCtx) << "Error during readLatest query: " << error.message << uid; });
 }
 
-void EntityStore::readLatest(const QByteArray &type, const QByteArray &uid, const std::function<void(const ApplicationDomain::ApplicationDomainType &)> callback)
+void EntityStore::readLatest(const QByteArray &type, const QByteArray &uid, const std::function<void(const ApplicationDomainType &)> callback)
 {
     readLatest(type, uid, [&](const QByteArray &uid, const EntityBuffer &buffer) {
         //TODO cache max revision for the duration of the transaction.
@@ -531,7 +499,7 @@ void EntityStore::readLatest(const QByteArray &type, const QByteArray &uid, cons
     });
 }
 
-void EntityStore::readLatest(const QByteArray &type, const QByteArray &uid, const std::function<void(const ApplicationDomain::ApplicationDomainType &, Sink::Operation)> callback)
+void EntityStore::readLatest(const QByteArray &type, const QByteArray &uid, const std::function<void(const ApplicationDomainType &, Sink::Operation)> callback)
 {
     readLatest(type, uid, [&](const QByteArray &uid, const EntityBuffer &buffer) {
         //TODO cache max revision for the duration of the transaction.
@@ -541,9 +509,9 @@ void EntityStore::readLatest(const QByteArray &type, const QByteArray &uid, cons
 
 ApplicationDomain::ApplicationDomainType EntityStore::readLatest(const QByteArray &type, const QByteArray &uid)
 {
-    ApplicationDomain::ApplicationDomainType dt;
-    readLatest(type, uid, [&](const ApplicationDomain::ApplicationDomainType &entity) {
-        dt = entity;
+    ApplicationDomainType dt;
+    readLatest(type, uid, [&](const ApplicationDomainType &entity) {
+        dt = *ApplicationDomainType::getInMemoryRepresentation<ApplicationDomainType>(entity, entity.availableProperties());
     });
     return dt;
 }
@@ -556,10 +524,10 @@ void EntityStore::readEntity(const QByteArray &type, const QByteArray &key, cons
             callback(DataStore::uidFromKey(key), Sink::EntityBuffer(value.data(), value.size()));
             return false;
         },
-        [&](const DataStore::Error &error) { SinkWarningCtx(d->logCtx) << "Error during query: " << error.message << key; });
+        [&](const DataStore::Error &error) { SinkWarningCtx(d->logCtx) << "Error during readEntity query: " << error.message << key; });
 }
 
-void EntityStore::readEntity(const QByteArray &type, const QByteArray &uid, const std::function<void(const ApplicationDomain::ApplicationDomainType &)> callback)
+void EntityStore::readEntity(const QByteArray &type, const QByteArray &uid, const std::function<void(const ApplicationDomainType &)> callback)
 {
     readEntity(type, uid, [&](const QByteArray &uid, const EntityBuffer &buffer) {
         callback(d->createApplicationDomainType(type, uid, DataStore::maxRevision(d->getTransaction()), buffer));
@@ -568,15 +536,15 @@ void EntityStore::readEntity(const QByteArray &type, const QByteArray &uid, cons
 
 ApplicationDomain::ApplicationDomainType EntityStore::readEntity(const QByteArray &type, const QByteArray &uid)
 {
-    ApplicationDomain::ApplicationDomainType dt;
-    readEntity(type, uid, [&](const ApplicationDomain::ApplicationDomainType &entity) {
-        dt = entity;
+    ApplicationDomainType dt;
+    readEntity(type, uid, [&](const ApplicationDomainType &entity) {
+        dt = *ApplicationDomainType::getInMemoryRepresentation<ApplicationDomainType>(entity, entity.availableProperties());
     });
     return dt;
 }
 
 
-void EntityStore::readAll(const QByteArray &type, const std::function<void(const ApplicationDomain::ApplicationDomainType &entity)> &callback)
+void EntityStore::readAll(const QByteArray &type, const std::function<void(const ApplicationDomainType &entity)> &callback)
 {
     readAllUids(type, [&] (const QByteArray &uid) {
         readLatest(type, uid, callback);
@@ -611,17 +579,17 @@ void EntityStore::readPrevious(const QByteArray &type, const QByteArray &uid, qi
     qint64 latestRevision = 0;
     db.scan(uid,
         [&latestRevision, revision](const QByteArray &key, const QByteArray &) -> bool {
-            const auto foundRevision = Sink::Storage::DataStore::revisionFromKey(key);
+            const auto foundRevision = DataStore::revisionFromKey(key);
             if (foundRevision < revision && foundRevision > latestRevision) {
                 latestRevision = foundRevision;
             }
             return true;
         },
-        [&](const Sink::Storage::DataStore::Error &error) { SinkWarningCtx(d->logCtx) << "Failed to read current value from storage: " << error.message; }, true);
-    return readEntity(type, Sink::Storage::DataStore::assembleKey(uid, latestRevision), callback);
+        [&](const DataStore::Error &error) { SinkWarningCtx(d->logCtx) << "Failed to read current value from storage: " << error.message; }, true);
+    readEntity(type, DataStore::assembleKey(uid, latestRevision), callback);
 }
 
-void EntityStore::readPrevious(const QByteArray &type, const QByteArray &uid, qint64 revision, const std::function<void(const ApplicationDomain::ApplicationDomainType &)> callback)
+void EntityStore::readPrevious(const QByteArray &type, const QByteArray &uid, qint64 revision, const std::function<void(const ApplicationDomainType &)> callback)
 {
     readPrevious(type, uid, revision, [&](const QByteArray &uid, const EntityBuffer &buffer) {
         callback(d->createApplicationDomainType(type, uid, DataStore::maxRevision(d->getTransaction()), buffer));
@@ -630,16 +598,16 @@ void EntityStore::readPrevious(const QByteArray &type, const QByteArray &uid, qi
 
 ApplicationDomain::ApplicationDomainType EntityStore::readPrevious(const QByteArray &type, const QByteArray &uid, qint64 revision)
 {
-    ApplicationDomain::ApplicationDomainType dt;
-    readPrevious(type, uid, revision, [&](const ApplicationDomain::ApplicationDomainType &entity) {
-        dt = entity;
+    ApplicationDomainType dt;
+    readPrevious(type, uid, revision, [&](const ApplicationDomainType &entity) {
+        dt = *ApplicationDomainType::getInMemoryRepresentation<ApplicationDomainType>(entity, entity.availableProperties());
     });
     return dt;
 }
 
 void EntityStore::readAllUids(const QByteArray &type, const std::function<void(const QByteArray &uid)> callback)
 {
-    DataStore::getUids(d->getTransaction(), callback);
+    DataStore::getUids(type, d->getTransaction(), callback);
 }
 
 bool EntityStore::contains(const QByteArray &type, const QByteArray &uid)
@@ -653,7 +621,7 @@ bool EntityStore::exists(const QByteArray &type, const QByteArray &uid)
     bool alreadyRemoved = false;
     DataStore::mainDatabase(d->transaction, type)
         .findLatest(uid,
-            [&found, &alreadyRemoved](const QByteArray &key, const QByteArray &data) -> bool {
+            [&found, &alreadyRemoved](const QByteArray &key, const QByteArray &data) {
                 auto entity = GetEntity(data.data());
                 if (entity && entity->metadata()) {
                     auto metadata = GetMetadata(entity->metadata()->Data());
@@ -662,7 +630,6 @@ bool EntityStore::exists(const QByteArray &type, const QByteArray &uid)
                         alreadyRemoved = true;
                     }
                 }
-                return false;
             },
             [&](const DataStore::Error &error) { SinkWarningCtx(d->logCtx) << "Failed to read old revision from storage: " << error.message; });
     if (!found) {
@@ -676,6 +643,22 @@ bool EntityStore::exists(const QByteArray &type, const QByteArray &uid)
     return true;
 }
 
+void EntityStore::readRevisions(const QByteArray &type, const QByteArray &uid, qint64 startingRevision, const std::function<void(const QByteArray &uid, qint64 revision, const EntityBuffer &entity)> callback)
+{
+    Q_ASSERT(d);
+    Q_ASSERT(!uid.isEmpty());
+    DataStore::mainDatabase(d->transaction, type)
+        .scan(uid,
+            [&](const QByteArray &key, const QByteArray &value) -> bool {
+                const auto revision = DataStore::revisionFromKey(key);
+                if (revision >= startingRevision) {
+                    callback(DataStore::uidFromKey(key), revision, Sink::EntityBuffer(value.data(), value.size()));
+                }
+                return true;
+            },
+            [&](const DataStore::Error &error) { SinkWarningCtx(d->logCtx) << "Error while reading: " << error.message; }, true);
+
+}
 
 qint64 EntityStore::maxRevision()
 {
