@@ -37,14 +37,19 @@ class SINK_EXPORT EntityStore
 public:
     typedef QSharedPointer<EntityStore> Ptr;
     EntityStore(const ResourceContext &resourceContext, const Sink::Log::Context &);
+    ~EntityStore() = default;
+
+    using ApplicationDomainType = ApplicationDomain::ApplicationDomainType;
+
+    void initialize();
 
     //Only the pipeline may call the following functions outside of tests
-    bool add(const QByteArray &type, ApplicationDomain::ApplicationDomainType newEntity, bool replayToSource);
-    bool modify(const QByteArray &type, const ApplicationDomain::ApplicationDomainType &diff, const QByteArrayList &deletions, bool replayToSource);
-    bool modify(const QByteArray &type, const ApplicationDomain::ApplicationDomainType &current, ApplicationDomain::ApplicationDomainType newEntity, bool replayToSource);
-    bool remove(const QByteArray &type, const ApplicationDomain::ApplicationDomainType &current, bool replayToSource);
+    bool add(const QByteArray &type, ApplicationDomainType newEntity, bool replayToSource);
+    bool modify(const QByteArray &type, const ApplicationDomainType &diff, const QByteArrayList &deletions, bool replayToSource);
+    bool modify(const QByteArray &type, const ApplicationDomainType &current, ApplicationDomainType newEntity, bool replayToSource);
+    bool remove(const QByteArray &type, const ApplicationDomainType &current, bool replayToSource);
     bool cleanupRevisions(qint64 revision);
-    ApplicationDomain::ApplicationDomainType applyDiff(const QByteArray &type, const ApplicationDomain::ApplicationDomainType &current, const ApplicationDomain::ApplicationDomainType &diff, const QByteArrayList &deletions) const;
+    ApplicationDomainType applyDiff(const QByteArray &type, const ApplicationDomainType &current, const ApplicationDomainType &diff, const QByteArrayList &deletions, const QSet<QByteArray> &excludeProperties = {}) const;
 
     void startTransaction(Sink::Storage::DataStore::AccessMode);
     void commitTransaction();
@@ -52,7 +57,7 @@ public:
     bool hasTransaction() const;
 
     QVector<QByteArray> fullScan(const QByteArray &type);
-    QVector<QByteArray> indexLookup(const QByteArray &type, const QueryBase &query, QSet<QByteArray> &appliedFilters, QByteArray &appliedSorting);
+    QVector<QByteArray> indexLookup(const QByteArray &type, const QueryBase &query, QSet<QByteArrayList> &appliedFilters, QByteArray &appliedSorting);
     QVector<QByteArray> indexLookup(const QByteArray &type, const QByteArray &property, const QVariant &value);
     void indexLookup(const QByteArray &type, const QByteArray &property, const QVariant &value, const std::function<void(const QByteArray &uid)> &callback);
     template<typename EntityType, typename PropertyType>
@@ -60,20 +65,27 @@ public:
         return indexLookup(ApplicationDomain::getTypeName<EntityType>(), PropertyType::name, value, callback);
     }
 
+    ///Returns the uid and buffer. Note that the memory only remains valid until the next operation or transaction end.
     void readLatest(const QByteArray &type, const QByteArray &uid, const std::function<void(const QByteArray &uid, const EntityBuffer &entity)> callback);
-    void readLatest(const QByteArray &type, const QByteArray &uid, const std::function<void(const ApplicationDomain::ApplicationDomainType &entity)> callback);
-    void readLatest(const QByteArray &type, const QByteArray &uid, const std::function<void(const ApplicationDomain::ApplicationDomainType &entity, Sink::Operation)> callback);
+    ///Returns an entity. Note that the memory only remains valid until the next operation or transaction end.
+    void readLatest(const QByteArray &type, const QByteArray &uid, const std::function<void(const ApplicationDomainType &entity)> callback);
+    ///Returns an entity and operation. Note that the memory only remains valid until the next operation or transaction end.
+    void readLatest(const QByteArray &type, const QByteArray &uid, const std::function<void(const ApplicationDomainType &entity, Sink::Operation)> callback);
 
-    ApplicationDomain::ApplicationDomainType readLatest(const QByteArray &type, const QByteArray &uid);
+    ///Returns a copy
+    ApplicationDomainType readLatest(const QByteArray &type, const QByteArray &uid);
 
     template<typename T>
     T readLatest(const QByteArray &uid) {
         return T(readLatest(ApplicationDomain::getTypeName<T>(), uid));
     }
 
+    ///Returns the uid and buffer. Note that the memory only remains valid until the next operation or transaction end.
     void readEntity(const QByteArray &type, const QByteArray &uid, const std::function<void(const QByteArray &uid, const EntityBuffer &entity)> callback);
-    void readEntity(const QByteArray &type, const QByteArray &uid, const std::function<void(const ApplicationDomain::ApplicationDomainType &entity)> callback);
-    ApplicationDomain::ApplicationDomainType readEntity(const QByteArray &type, const QByteArray &key);
+    ///Returns an entity. Note that the memory only remains valid until the next operation or transaction end.
+    void readEntity(const QByteArray &type, const QByteArray &uid, const std::function<void(const ApplicationDomainType &entity)> callback);
+    ///Returns a copy
+    ApplicationDomainType readEntity(const QByteArray &type, const QByteArray &key);
 
     template<typename T>
     T readEntity(const QByteArray &key) {
@@ -82,8 +94,9 @@ public:
 
 
     void readPrevious(const QByteArray &type, const QByteArray &uid, qint64 revision, const std::function<void(const QByteArray &uid, const EntityBuffer &entity)> callback);
-    void readPrevious(const QByteArray &type, const QByteArray &uid, qint64 revision, const std::function<void(const ApplicationDomain::ApplicationDomainType &entity)> callback);
-    ApplicationDomain::ApplicationDomainType readPrevious(const QByteArray &type, const QByteArray &uid, qint64 revision);
+    void readPrevious(const QByteArray &type, const QByteArray &uid, qint64 revision, const std::function<void(const ApplicationDomainType &entity)> callback);
+    ///Returns a copy
+    ApplicationDomainType readPrevious(const QByteArray &type, const QByteArray &uid, qint64 revision);
 
     template<typename T>
     T readPrevious(const QByteArray &uid, qint64 revision) {
@@ -92,11 +105,11 @@ public:
 
     void readAllUids(const QByteArray &type, const std::function<void(const QByteArray &uid)> callback);
 
-    void readAll(const QByteArray &type, const std::function<void(const ApplicationDomain::ApplicationDomainType &entity)> &callback);
+    void readAll(const QByteArray &type, const std::function<void(const ApplicationDomainType &entity)> &callback);
 
     template<typename T>
     void readAll(const std::function<void(const T &entity)> &callback) {
-        return readAll(ApplicationDomain::getTypeName<T>(), [&](const ApplicationDomain::ApplicationDomainType &entity) {
+        return readAll(ApplicationDomain::getTypeName<T>(), [&](const ApplicationDomainType &entity) {
             callback(T(entity));
         });
     }
@@ -109,6 +122,8 @@ public:
     ///Db contains entity and entity is not yet removed
     bool exists(const QByteArray &type, const QByteArray &uid);
 
+    void readRevisions(const QByteArray &type, const QByteArray &uid, qint64 baseRevision, const std::function<void(const QByteArray &uid, qint64 revision, const EntityBuffer &entity)> callback);
+
     qint64 maxRevision();
 
     Sink::Log::Context logContext() const;
@@ -118,7 +133,7 @@ private:
      * Remove any old revisions of the same entity up until @param revision
      */
     void cleanupEntityRevisionsUntil(qint64 revision);
-    void copyBlobs(ApplicationDomain::ApplicationDomainType &entity, qint64 newRevision);
+    void copyBlobs(ApplicationDomainType &entity, qint64 newRevision);
     class Private;
     const QSharedPointer<Private> d;
 };
