@@ -22,7 +22,6 @@
 
 #include <QTime>
 #include <QAbstractItemModel>
-#include <QUuid>
 #include <functional>
 #include <memory>
 
@@ -35,11 +34,36 @@
 #include "modelresult.h"
 #include "storage.h"
 #include "log.h"
+#include "utils.h"
+
+#define ASSERT_ENUMS_MATCH(A, B) Q_STATIC_ASSERT_X(static_cast<int>(A) == static_cast<int>(B), "The enum values must match");
+
+//Ensure the copied enum matches
+typedef ModelResult<Sink::ApplicationDomain::Mail, Sink::ApplicationDomain::Mail::Ptr> MailModelResult;
+ASSERT_ENUMS_MATCH(Sink::Store::DomainObjectBaseRole, MailModelResult::DomainObjectBaseRole)
+ASSERT_ENUMS_MATCH(Sink::Store::ChildrenFetchedRole, MailModelResult::ChildrenFetchedRole)
+ASSERT_ENUMS_MATCH(Sink::Store::DomainObjectRole, MailModelResult::DomainObjectRole)
+ASSERT_ENUMS_MATCH(Sink::Store::StatusRole, MailModelResult::StatusRole)
+ASSERT_ENUMS_MATCH(Sink::Store::WarningRole, MailModelResult::WarningRole)
+ASSERT_ENUMS_MATCH(Sink::Store::ProgressRole, MailModelResult::ProgressRole)
 
 Q_DECLARE_METATYPE(QSharedPointer<Sink::ResultEmitter<Sink::ApplicationDomain::SinkResource::Ptr>>)
 Q_DECLARE_METATYPE(QSharedPointer<Sink::ResourceAccessInterface>);
 Q_DECLARE_METATYPE(std::shared_ptr<void>);
 
+
+static bool sanityCheckQuery(const Sink::Query &query)
+{
+    for (const auto &id : query.ids()) {
+        if (id.isEmpty()) {
+            SinkError() << "Empty id in query.";
+            return false;
+        }
+    }
+    return true;
+}
+
+
 namespace Sink {
 
 QString Store::storageLocation()
@@ -47,11 +71,6 @@ QString Store::storageLocation()
     return Sink::storageLocation();
 }
 
-QString Store::getTemporaryFilePath()
-{
-    return Sink::temporaryFileLocation() + "/" + QUuid::createUuid().toString();
-}
-
 
 template <class DomainType>
 KAsync::Job<void> queryResource(const QByteArray resourceType, const QByteArray &resourceInstanceIdentifier, const Query &query, typename AggregatingResultEmitter<typename DomainType::Ptr>::Ptr aggregatingEmitter, const Sink::Log::Context &ctx_)
@@ -98,11 +117,13 @@ QPair<typename AggregatingResultEmitter<typename DomainType::Ptr>::Ptr,  typenam
 
         //Filter resources by available content types (unless the query already specifies a capability filter)
         auto resourceFilter = query.getResourceFilter();
-        if (!resourceFilter.propertyFilter.contains(ApplicationDomain::SinkResource::Capabilities::name)) {
-            resourceFilter.propertyFilter.insert(ApplicationDomain::SinkResource::Capabilities::name, Query::Comparator{ApplicationDomain::getTypeName<DomainType>(), Query::Comparator::Contains});
+        if (!resourceFilter.propertyFilter.contains({ApplicationDomain::SinkResource::Capabilities::name})) {
+            resourceFilter.propertyFilter.insert({ApplicationDomain::SinkResource::Capabilities::name}, Query::Comparator{ApplicationDomain::getTypeName<DomainType>(), Query::Comparator::Contains});
         }
         resourceQuery.setFilter(resourceFilter);
-        resourceQuery.requestedProperties << resourceFilter.propertyFilter.keys();
+        for (auto const &properties :  resourceFilter.propertyFilter.keys()) {
+            resourceQuery.requestedProperties << properties;
+        }
 
         auto result = facade->load(resourceQuery, resourceCtx);
         auto emitter = result.second;
@@ -112,12 +133,6 @@ QPair<typename AggregatingResultEmitter<typename DomainType::Ptr>::Ptr,  typenam
             Q_ASSERT(!resourceType.isEmpty());
             queryResource<DomainType>(resourceType, resource->identifier(), query, aggregatingEmitter, ctx).exec();
         });
-        emitter->onModified([](const ApplicationDomain::SinkResource::Ptr &) {
-        });
-        emitter->onRemoved([](const ApplicationDomain::SinkResource::Ptr &) {
-        });
-        emitter->onInitialResultSetComplete([](const ApplicationDomain::SinkResource::Ptr &, bool) {
-        });
         emitter->onComplete([query, aggregatingEmitter, resourceCtx]() {
             SinkTraceCtx(resourceCtx) << "Resource query complete";
         });
@@ -138,6 +153,7 @@ static Log::Context getQueryContext(const Sink::Query &query, const QByteArray &
 template <class DomainType>
 QSharedPointer<QAbstractItemModel> Store::loadModel(const Query &query)
 {
+    Q_ASSERT(sanityCheckQuery(query));
     auto ctx = getQueryContext(query, ApplicationDomain::getTypeName<DomainType>());
     auto model = QSharedPointer<ModelResult<DomainType, typename DomainType::Ptr>>::create(query, query.requestedProperties, ctx);
 
@@ -153,7 +169,7 @@ QSharedPointer<QAbstractItemModel> Store::loadModel(const Query &query)
     //Keep the emitter alive
     if (auto resourceEmitter = result.second) {
         model->setProperty("resourceEmitter", QVariant::fromValue(resourceEmitter)); //TODO only neceesary for live queries
-        resourceEmitter->fetch(ApplicationDomain::SinkResource::Ptr());
+        resourceEmitter->fetch();
     }
 
 
@@ -181,23 +197,37 @@ template <class DomainType>
 KAsync::Job<void> Store::create(const DomainType &domainObject)
 {
     SinkLog() << "Create: " << domainObject;
-    // Potentially move to separate thread as well
     auto facade = getFacade<DomainType>(domainObject.resourceInstanceIdentifier());
-    return facade->create(domainObject).addToContext(std::shared_ptr<void>(facade)).onError([](const KAsync::Error &error) { SinkWarning() << "Failed to create"; });
+    return facade->create(domainObject).addToContext(std::shared_ptr<void>(facade)).onError([](const KAsync::Error &error) { SinkWarning() << "Failed to create " << error; });
 }
 
 template <class DomainType>
 KAsync::Job<void> Store::modify(const DomainType &domainObject)
 {
+    if (domainObject.changedProperties().isEmpty()) {
+        SinkLog() << "Nothing to modify: " << domainObject.identifier();
+        return KAsync::null();
+    }
     SinkLog() << "Modify: " << domainObject;
-    // Potentially move to separate thread as well
     auto facade = getFacade<DomainType>(domainObject.resourceInstanceIdentifier());
+    if (domainObject.isAggregate()) {
+        return KAsync::value(domainObject.aggregatedIds())
+            .addToContext(std::shared_ptr<void>(facade))
+            .each([=] (const QByteArray &id) {
+                auto object = Sink::ApplicationDomain::ApplicationDomainType::createCopy(id, domainObject);
+                return facade->modify(object).onError([](const KAsync::Error &error) { SinkWarning() << "Failed to modify " << error; });
+        });
+    }
     return facade->modify(domainObject).addToContext(std::shared_ptr<void>(facade)).onError([](const KAsync::Error &error) { SinkWarning() << "Failed to modify"; });
 }
 
 template <class DomainType>
 KAsync::Job<void> Store::modify(const Query &query, const DomainType &domainObject)
 {
+    if (domainObject.changedProperties().isEmpty()) {
+        SinkLog() << "Nothing to modify: " << domainObject.identifier();
+        return KAsync::null();
+    }
     SinkLog() << "Modify: " << query << domainObject;
     return fetchAll<DomainType>(query)
         .each([=] (const typename DomainType::Ptr &entity) {
@@ -213,27 +243,48 @@ template <class DomainType>
 KAsync::Job<void> Store::move(const DomainType &domainObject, const QByteArray &newResource)
 {
     SinkLog() << "Move: " << domainObject << newResource;
-    // Potentially move to separate thread as well
     auto facade = getFacade<DomainType>(domainObject.resourceInstanceIdentifier());
-    return facade->move(domainObject, newResource).addToContext(std::shared_ptr<void>(facade)).onError([](const KAsync::Error &error) { SinkWarning() << "Failed to move"; });
+    if (domainObject.isAggregate()) {
+        return KAsync::value(domainObject.aggregatedIds())
+            .addToContext(std::shared_ptr<void>(facade))
+            .each([=] (const QByteArray &id) {
+                auto object = Sink::ApplicationDomain::ApplicationDomainType::createCopy(id, domainObject);
+                return facade->move(object, newResource).onError([](const KAsync::Error &error) { SinkWarning() << "Failed to move " << error; });
+        });
+    }
+    return facade->move(domainObject, newResource).addToContext(std::shared_ptr<void>(facade)).onError([](const KAsync::Error &error) { SinkWarning() << "Failed to move " << error; });
 }
 
 template <class DomainType>
 KAsync::Job<void> Store::copy(const DomainType &domainObject, const QByteArray &newResource)
 {
     SinkLog() << "Copy: " << domainObject << newResource;
-    // Potentially copy to separate thread as well
     auto facade = getFacade<DomainType>(domainObject.resourceInstanceIdentifier());
-    return facade->copy(domainObject, newResource).addToContext(std::shared_ptr<void>(facade)).onError([](const KAsync::Error &error) { SinkWarning() << "Failed to copy"; });
+    if (domainObject.isAggregate()) {
+        return KAsync::value(domainObject.aggregatedIds())
+            .addToContext(std::shared_ptr<void>(facade))
+            .each([=] (const QByteArray &id) {
+                auto object = Sink::ApplicationDomain::ApplicationDomainType::createCopy(id, domainObject);
+                return facade->copy(object, newResource).onError([](const KAsync::Error &error) { SinkWarning() << "Failed to copy " << error; });
+        });
+    }
+    return facade->copy(domainObject, newResource).addToContext(std::shared_ptr<void>(facade)).onError([](const KAsync::Error &error) { SinkWarning() << "Failed to copy " << error; });
 }
 
 template <class DomainType>
 KAsync::Job<void> Store::remove(const DomainType &domainObject)
 {
     SinkLog() << "Remove: " << domainObject;
-    // Potentially move to separate thread as well
     auto facade = getFacade<DomainType>(domainObject.resourceInstanceIdentifier());
-    return facade->remove(domainObject).addToContext(std::shared_ptr<void>(facade)).onError([](const KAsync::Error &error) { SinkWarning() << "Failed to remove"; });
+    if (domainObject.isAggregate()) {
+        return KAsync::value(domainObject.aggregatedIds())
+            .addToContext(std::shared_ptr<void>(facade))
+            .each([=] (const QByteArray &id) {
+                auto object = Sink::ApplicationDomain::ApplicationDomainType::createCopy(id, domainObject);
+                return facade->remove(object).onError([](const KAsync::Error &error) { SinkWarning() << "Failed to remove " << error; });
+        });
+    }
+    return facade->remove(domainObject).addToContext(std::shared_ptr<void>(facade)).onError([](const KAsync::Error &error) { SinkWarning() << "Failed to remove " << error; });
 }
 
 template <class DomainType>
@@ -275,16 +326,57 @@ KAsync::Job<void> Store::removeDataFromDisk(const QByteArray &identifier)
         });
 }
 
-KAsync::Job<void> Store::upgrade()
+static KAsync::Job<Store::UpgradeResult> upgrade(const QByteArray &resource)
+{
+    auto store = Sink::Storage::DataStore(Sink::storageLocation(), resource, Sink::Storage::DataStore::ReadOnly);
+    if (!store.exists() || Storage::DataStore::databaseVersion(store.createTransaction(Storage::DataStore::ReadOnly)) == Sink::latestDatabaseVersion()) {
+        return KAsync::value(Store::UpgradeResult{false});
+    }
+    SinkLog() << "Upgrading " << resource;
+
+    //We're not using the factory to avoid getting a cached resourceaccess with the wrong resourceType
+    auto resourceAccess = Sink::ResourceAccess::Ptr{new Sink::ResourceAccess(resource, ResourceConfig::getResourceType(resource)), &QObject::deleteLater};
+    return resourceAccess->sendCommand(Sink::Commands::UpgradeCommand)
+        .addToContext(resourceAccess)
+        .then([=](const KAsync::Error &error) {
+            if (error) {
+                SinkWarning() << "Error during upgrade.";
+                return KAsync::error(error);
+            }
+            SinkTrace() << "Upgrade of resource " << resource << " complete.";
+            return KAsync::null();
+        })
+        .then(KAsync::value(Store::UpgradeResult{true}));
+}
+
+KAsync::Job<Store::UpgradeResult> Store::upgrade()
 {
     SinkLog() << "Upgrading...";
+
+    //Migrate from sink.dav to sink.carddav
+    const auto resources = ResourceConfig::getResources();
+    for (auto it = resources.constBegin(); it != resources.constEnd(); it++) {
+        if (it.value() == "sink.dav") {
+            ResourceConfig::setResourceType(it.key(), "sink.carddav");
+        }
+    }
+
+    auto ret = QSharedPointer<bool>::create(false);
     return fetchAll<ApplicationDomain::SinkResource>({})
-        .template each([](const ApplicationDomain::SinkResource::Ptr &resource) -> KAsync::Job<void> {
-            SinkLog() << "Removing caches for " << resource->identifier();
-            return removeDataFromDisk(resource->identifier());
+        .template each([ret](const ApplicationDomain::SinkResource::Ptr &resource) -> KAsync::Job<void> {
+            return Sink::upgrade(resource->identifier())
+                .then([ret](UpgradeResult returnValue) {
+                    if (returnValue.upgradeExecuted) {
+                        SinkLog() << "Upgrade executed.";
+                        *ret = true;
+                    }
+                });
         })
-        .then([] {
-            SinkLog() << "Upgrade complete.";
+        .then([ret] {
+            if (*ret) {
+                SinkLog() << "Upgrade complete.";
+            }
+            return Store::UpgradeResult{*ret};
         });
 }
 
@@ -311,9 +403,14 @@ KAsync::Job<void> Store::synchronize(const Sink::Query &query)
 
 KAsync::Job<void> Store::synchronize(const Sink::SyncScope &scope)
 {
+    auto resourceFilter = scope.getResourceFilter();
+    //Filter resources by type by default
+    if (!resourceFilter.propertyFilter.contains({ApplicationDomain::SinkResource::Capabilities::name}) && !scope.type().isEmpty()) {
+        resourceFilter.propertyFilter.insert({ApplicationDomain::SinkResource::Capabilities::name}, Query::Comparator{scope.type(), Query::Comparator::Contains});
+    }
     Sink::Query query;
-    query.setFilter(scope.getResourceFilter());
-    SinkLog() << "Synchronizing: " << query;
+    query.setFilter(resourceFilter);
+    SinkLog() << "Synchronizing all resource matching: " << query;
     return fetchAll<ApplicationDomain::SinkResource>(query)
         .template each([scope](const ApplicationDomain::SinkResource::Ptr &resource) -> KAsync::Job<void> {
             return synchronize(resource->identifier(), scope);
@@ -337,6 +434,7 @@ KAsync::Job<QList<typename DomainType::Ptr>> Store::fetchAll(const Sink::Query &
 template <class DomainType>
 KAsync::Job<QList<typename DomainType::Ptr>> Store::fetch(const Sink::Query &query, int minimumAmount)
 {
+    Q_ASSERT(sanityCheckQuery(query));
     auto model = loadModel<DomainType>(query);
     auto list = QSharedPointer<QList<typename DomainType::Ptr>>::create();
     auto context = QSharedPointer<QObject>::create();
@@ -346,7 +444,7 @@ KAsync::Job<QList<typename DomainType::Ptr>> Store::fetch(const Sink::Query &que
                 list->append(model->index(i, 0, QModelIndex()).data(Sink::Store::DomainObjectRole).template value<typename DomainType::Ptr>());
             }
         } else {
-            QObject::connect(model.data(), &QAbstractItemModel::rowsInserted, context.data(), [model, &future, list](const QModelIndex &index, int start, int end) {
+            QObject::connect(model.data(), &QAbstractItemModel::rowsInserted, context.data(), [model, list](const QModelIndex &index, int start, int end) {
                 for (int i = start; i <= end; i++) {
                     list->append(model->index(i, 0, QModelIndex()).data(Sink::Store::DomainObjectRole).template value<typename DomainType::Ptr>());
                 }
@@ -388,6 +486,7 @@ DomainType Store::readOne(const Sink::Query &query)
 template <class DomainType>
 QList<DomainType> Store::read(const Sink::Query &query_)
 {
+    Q_ASSERT(sanityCheckQuery(query_));
     auto query = query_;
     query.setFlags(Query::SynchronousQuery);
 
@@ -403,10 +502,10 @@ QList<DomainType> Store::read(const Sink::Query &query_)
     });
 
     if (auto resourceEmitter = result.second) {
-        resourceEmitter->fetch(ApplicationDomain::SinkResource::Ptr());
+        resourceEmitter->fetch();
     }
 
-    aggregatingEmitter->fetch(typename DomainType::Ptr());
+    aggregatingEmitter->fetch();
     return list;
 }
 
