@@ -29,6 +29,7 @@
 #include "deleteentity_generated.h"
 #include "flush_generated.h"
 #include "notification_generated.h"
+#include "utils.h"
 
 using namespace Sink;
 
@@ -40,7 +41,7 @@ Synchronizer::Synchronizer(const Sink::ResourceContext &context)
     mSyncStorage(Sink::storageLocation(), mResourceContext.instanceId() + ".synchronization", Sink::Storage::DataStore::DataStore::ReadWrite),
     mSyncInProgress(false)
 {
-    mCurrentState.push(ApplicationDomain::Status::OfflineStatus);
+    mCurrentState.push(ApplicationDomain::Status::NoStatus);
     SinkTraceCtx(mLogCtx) << "Starting synchronizer: " << mResourceContext.resourceType << mResourceContext.instanceId();
 }
 
@@ -49,6 +50,20 @@ Synchronizer::~Synchronizer()
 
 }
 
+void Synchronizer::setSecret(const QString &s)
+{
+    mSecret = s;
+
+    if (!mSyncRequestQueue.isEmpty()) {
+        processSyncQueue().exec();
+    }
+}
+
+QString Synchronizer::secret() const
+{
+    return mSecret;
+}
+
 void Synchronizer::setup(const std::function<void(int commandId, const QByteArray &data)> &enqueueCommandCallback, MessageQueue &mq)
 {
     mEnqueue = enqueueCommandCallback;
@@ -169,7 +184,11 @@ void Synchronizer::modifyIfChanged(Storage::EntityStore &store, const QByteArray
 
 void Synchronizer::modify(const QByteArray &bufferType, const QByteArray &remoteId, const Sink::ApplicationDomain::ApplicationDomainType &entity)
 {
-    const auto sinkId = syncStore().resolveRemoteId(bufferType, remoteId);
+    const auto sinkId = syncStore().resolveRemoteId(bufferType, remoteId, false);
+    if (sinkId.isEmpty()) {
+        SinkWarningCtx(mLogCtx) << "Can't modify entity that is not locally existing " << remoteId;
+        return;
+    }
     Storage::EntityStore store(mResourceContext, mLogCtx);
     modifyIfChanged(store, bufferType, sinkId, entity);
 }
@@ -229,13 +248,18 @@ QByteArrayList Synchronizer::resolveFilter(const QueryBase::Comparator &filter)
 {
     QByteArrayList result;
     if (filter.value.canConvert<QByteArray>()) {
-        result << filter.value.value<QByteArray>();
+        const auto value = filter.value.value<QByteArray>();
+        if (value.isEmpty()) {
+            SinkErrorCtx(mLogCtx) << "Tried to filter for an empty value: " << filter;
+        } else {
+            result << filter.value.value<QByteArray>();
+        }
     } else if (filter.value.canConvert<QueryBase>()) {
         auto query = filter.value.value<QueryBase>();
         Storage::EntityStore store{mResourceContext, mLogCtx};
         DataStoreQuery dataStoreQuery{query, query.type(), store};
         auto resultSet = dataStoreQuery.execute();
-        resultSet.replaySet(0, 0, [this, &result](const ResultSet::Result &r) {
+        resultSet.replaySet(0, 0, [&result](const ResultSet::Result &r) {
             result << r.entity.identifier();
         });
     } else {
@@ -318,6 +342,10 @@ void Synchronizer::emitProgressNotification(Notification::NoticationType type, i
 void Synchronizer::reportProgress(int progress, int total, const QByteArrayList &entities)
 {
     if (progress > 0 && total > 0) {
+        //Limit progress updates for large amounts
+        if (total >= 100 && progress % 10 != 0) {
+            return;
+        }
         SinkLogCtx(mLogCtx) << "Progress: " << progress << " out of " << total << mCurrentRequest.requestId << mCurrentRequest.applicableEntities;
         const auto applicableEntities = [&] {
             if (entities.isEmpty()) {
@@ -344,8 +372,11 @@ void Synchronizer::setStatusFromResult(const KAsync::Error &error, const QString
         } else if (error.errorCode == ApplicationDomain::LoginError) {
             //If we failed to login altough we could connect that indicates a problem with our setup.
             setStatus(ApplicationDomain::ErrorStatus, s, requestId);
+        } else if (error.errorCode == ApplicationDomain::ConnectionLostError) {
+            //We've lost the connection so we assume the connection to the server broke.
+            setStatus(ApplicationDomain::OfflineStatus, s, requestId);
         }
-        //We don't know what kind of error this was, so we assume it's transient and don't change ou status.
+        //We don't know what kind of error this was, so we assume it's transient and don't change our status.
     } else {
         //An operation against the server worked, so we're probably online.
         setStatus(ApplicationDomain::ConnectedStatus, s, requestId);
@@ -361,7 +392,7 @@ KAsync::Job<void> Synchronizer::processRequest(const SyncRequest &request)
             modifiedRequest.options = SyncRequest::NoOptions;
             //Normally we won't have a requestId here
             if (modifiedRequest.requestId.isEmpty()) {
-                modifiedRequest.requestId = QUuid::createUuid().toByteArray();
+                modifiedRequest.requestId = createUuid();
             }
             SinkTraceCtx(mLogCtx) << "Enqueuing flush request " << modifiedRequest.requestId;
 
@@ -467,6 +498,11 @@ void Synchronizer::setBusy(bool busy, const QString &reason, const QByteArray re
 
 KAsync::Job<void> Synchronizer::processSyncQueue()
 {
+    if (secret().isEmpty()) {
+        SinkWarningCtx(mLogCtx) << "Secret not available but required.";
+        emitNotification(Notification::Warning, ApplicationDomain::SyncError, "Secret is not available.", {}, {});
+        return KAsync::null<void>();
+    }
     if (mSyncRequestQueue.isEmpty()) {
         SinkLogCtx(mLogCtx) << "All requests processed.";
         return KAsync::null<void>();
@@ -590,20 +626,35 @@ KAsync::Job<void> Synchronizer::replay(const QByteArray &type, const QByteArray
     }
     SinkLogCtx(mLogCtx) << "Replaying: " << key << "Type: " << type << "Uid: " << uid << "Rid: " << oldRemoteId << "Revision: " << metadataBuffer->revision();
 
+    //If the entity has been removed already and this is not the removal, skip over.
+    //This is important so we can unblock changereplay by removing entities.
+    bool skipOver = false;
+    store().readLatest(type, uid, [&](const ApplicationDomain::ApplicationDomainType &, Sink::Operation latestOperation) {
+        if (latestOperation == Sink::Operation_Removal && operation != Sink::Operation_Removal) {
+            skipOver = true;
+        }
+    });
+    if (skipOver) {
+        SinkLogCtx(mLogCtx) << "Skipping over already removed entity";
+        return KAsync::null();
+    }
+
     KAsync::Job<QByteArray> job = KAsync::null<QByteArray>();
     //TODO This requires supporting every domain type here as well. Can we solve this better so we can do the dispatch somewhere centrally?
     if (type == ApplicationDomain::getTypeName<ApplicationDomain::Folder>()) {
-        auto folder = store().readEntity<ApplicationDomain::Folder>(key);
-        job = replay(folder, operation, oldRemoteId, modifiedProperties);
+        job = replay(store().readEntity<ApplicationDomain::Folder>(key), operation, oldRemoteId, modifiedProperties);
     } else if (type == ApplicationDomain::getTypeName<ApplicationDomain::Mail>()) {
-        auto mail = store().readEntity<ApplicationDomain::Mail>(key);
-        job = replay(mail, operation, oldRemoteId, modifiedProperties);
+        job = replay(store().readEntity<ApplicationDomain::Mail>(key), operation, oldRemoteId, modifiedProperties);
     } else if (type == ApplicationDomain::getTypeName<ApplicationDomain::Contact>()) {
-        auto mail = store().readEntity<ApplicationDomain::Contact>(key);
-        job = replay(mail, operation, oldRemoteId, modifiedProperties);
+        job = replay(store().readEntity<ApplicationDomain::Contact>(key), operation, oldRemoteId, modifiedProperties);
     } else if (type == ApplicationDomain::getTypeName<ApplicationDomain::Addressbook>()) {
-        auto mail = store().readEntity<ApplicationDomain::Addressbook>(key);
-        job = replay(mail, operation, oldRemoteId, modifiedProperties);
+        job = replay(store().readEntity<ApplicationDomain::Addressbook>(key), operation, oldRemoteId, modifiedProperties);
+    } else if (type == ApplicationDomain::getTypeName<ApplicationDomain::Event>()) {
+        job = replay(store().readEntity<ApplicationDomain::Event>(key), operation, oldRemoteId, modifiedProperties);
+    } else if (type == ApplicationDomain::getTypeName<ApplicationDomain::Todo>()) {
+        job = replay(store().readEntity<ApplicationDomain::Todo>(key), operation, oldRemoteId, modifiedProperties);
+    } else if (type == ApplicationDomain::getTypeName<ApplicationDomain::Calendar>()) {
+        job = replay(store().readEntity<ApplicationDomain::Calendar>(key), operation, oldRemoteId, modifiedProperties);
     } else {
         SinkErrorCtx(mLogCtx) << "Replayed unknown type: " << type;
     }
@@ -660,6 +711,21 @@ KAsync::Job<QByteArray> Synchronizer::replay(const ApplicationDomain::Folder &,
     return KAsync::null<QByteArray>();
 }
 
+KAsync::Job<QByteArray> Synchronizer::replay(const ApplicationDomain::Event &, Sink::Operation, const QByteArray &, const QList<QByteArray> &)
+{
+    return KAsync::null<QByteArray>();
+}
+
+KAsync::Job<QByteArray> Synchronizer::replay(const ApplicationDomain::Todo &, Sink::Operation, const QByteArray &, const QList<QByteArray> &)
+{
+    return KAsync::null<QByteArray>();
+}
+
+KAsync::Job<QByteArray> Synchronizer::replay(const ApplicationDomain::Calendar &, Sink::Operation, const QByteArray &, const QList<QByteArray> &)
+{
+    return KAsync::null<QByteArray>();
+}
+
 bool Synchronizer::allChangesReplayed()
 {
     if (!mSyncRequestQueue.isEmpty()) {
@@ -675,8 +741,3 @@ bool Synchronizer::allChangesReplayed()
 
 SINK_REGISTER_TYPES()
 
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wundefined-reinterpret-cast"
-#include "synchronizer.moc"
-#include "moc_synchronizer.cpp"
-#pragma clang diagnostic pop
