@@ -113,7 +113,7 @@ public:
         });
         return facade;
     }
-    ~TestFacade(){};
+    ~TestFacade() Q_DECL_OVERRIDE {};
     KAsync::Job<void> create(const T &domainObject) Q_DECL_OVERRIDE
     {
         mTestAccount->addEntity<T>(T::Ptr::create(domainObject));
@@ -150,25 +150,15 @@ public:
         // We have to do it this way, otherwise we're not setting the fetcher right
         auto emitter = resultProvider->emitter();
 
-        resultProvider->setFetcher([query, resultProvider, this](const typename T::Ptr &parent) {
-            if (parent) {
-                SinkTrace() << "Running the fetcher " << parent->identifier();
-            } else {
-                SinkTrace() << "Running the fetcher.";
-            }
+        resultProvider->setFetcher([query, resultProvider, this]() {
+            SinkTrace() << "Running the fetcher.";
             SinkTrace() << "-------------------------.";
             for (const auto &res : mTestAccount->entities<T>()) {
-                qDebug() << "Parent filter " << query.getFilter("parent").value.toByteArray() << res->identifier() << res->getProperty("parent").toByteArray();
-                auto parentProperty = res->getProperty("parent").toByteArray();
-                if ((!parent && parentProperty.isEmpty()) || (parent && parentProperty == parent->identifier()) || query.parentProperty().isEmpty()) {
-                    qDebug() << "Found a match" << res->identifier();
-                    resultProvider->add(res.template staticCast<T>());
-                }
+                resultProvider->add(res.template staticCast<T>());
             }
-            resultProvider->initialResultSetComplete(parent, true);
+            resultProvider->initialResultSetComplete(true);
         });
-        auto job = KAsync::start([query, resultProvider]() {});
-        return qMakePair(job, emitter);
+        return qMakePair(KAsync::null(), emitter);
     }
 
     Test::TestAccount *mTestAccount;
