@@ -145,19 +145,23 @@ public:
         return remoteId;
     }
 
+    static bool contains(const QVector<Folder> &folderList, const QByteArray &remoteId)
+    {
+        for (const auto &folder : folderList) {
+            if (folderRid(folder) == remoteId) {
+                return true;
+            }
+        }
+        return false;
+    }
+
     void synchronizeFolders(const QVector<Folder> &folderList)
     {
         SinkTraceCtx(mLogCtx) << "Found folders " << folderList.size();
 
         scanForRemovals(ENTITY_TYPE_FOLDER,
             [&folderList](const QByteArray &remoteId) -> bool {
-                // folderList.contains(remoteId)
-                for (const auto &folder : folderList) {
-                    if (folderRid(folder) == remoteId) {
-                        return true;
-                    }
-                }
-                return false;
+                return contains(folderList, remoteId);
             }
         );
 
@@ -246,17 +250,30 @@ public:
             SinkWarningCtx(mLogCtx) << "Invalid folder " << folderRemoteId << folder.path();
             return KAsync::error<void>("Invalid folder");
         }
-        // auto capabilities = imap->getCapabilities();
 
-        //First we fetch flag changes for all messages. Since we don't know which messages are locally available we just get everything and only apply to what we have.
-        return KAsync::start<void>([=]() {
-            auto uidNext = syncStore().readValue(folderRemoteId, "uidnext").toLongLong();
+        //Start by checking if UIDVALIDITY is still correct
+        return KAsync::start([=] {
+            bool ok = false;
+            const auto uidvalidity = syncStore().readValue(folderRemoteId, "uidvalidity").toLongLong(&ok);
+            return imap->select(folder)
+                .then([=](const SelectResult &selectResult) {
+                    SinkLogCtx(mLogCtx) << "Checking UIDVALIDITY. Local" << uidvalidity << "remote " << selectResult.uidValidity;
+                    if (ok && selectResult.uidValidity != uidvalidity) {
+                        SinkWarningCtx(mLogCtx) << "UIDVALIDITY changed " << selectResult.uidValidity << uidvalidity;
+                        syncStore().removePrefix(folderRemoteId);
+                    }
+                    syncStore().writeValue(folderRemoteId, "uidvalidity", QByteArray::number(selectResult.uidValidity));
+                });
+        })
+        // //First we fetch flag changes for all messages. Since we don't know which messages are locally available we just get everything and only apply to what we have.
+        .then([=] {
+            auto lastSeenUid = syncStore().readValue(folderRemoteId, "uidnext").toLongLong();
             bool ok = false;
             const auto changedsince = syncStore().readValue(folderRemoteId, "changedsince").toLongLong(&ok);
             SinkLogCtx(mLogCtx) << "About to update flags" << folder.path() << "changedsince: " << changedsince;
             //If we have any mails so far we start off by updating any changed flags using changedsince
             if (ok) {
-                return imap->fetchFlags(folder, KIMAP2::ImapSet(1, qMax(uidNext, qint64(1))), changedsince, [=](const Message &message) {
+                return imap->fetchFlags(folder, KIMAP2::ImapSet(1, qMax(lastSeenUid, qint64(1))), changedsince, [=](const Message &message) {
                     const auto folderLocalId = syncStore().resolveRemoteId(ENTITY_TYPE_FOLDER, folderRemoteId);
                     const auto remoteId = assembleMailRid(folderLocalId, message.uid);
 
@@ -270,6 +287,7 @@ public:
                 .then([=](const SelectResult &selectResult) {
                     SinkLogCtx(mLogCtx) << "Flags updated. New changedsince value: " << selectResult.highestModSequence;
                     syncStore().writeValue(folderRemoteId, "changedsince", QByteArray::number(selectResult.highestModSequence));
+                    return selectResult.uidNext;
                 });
             } else {
                 //We hit this path on initial sync and simply record the current changedsince value
@@ -277,30 +295,32 @@ public:
                 .then([=](const SelectResult &selectResult) {
                     SinkLogCtx(mLogCtx) << "No flags to update. New changedsince value: " << selectResult.highestModSequence;
                     syncStore().writeValue(folderRemoteId, "changedsince", QByteArray::number(selectResult.highestModSequence));
+                    return selectResult.uidNext;
                 });
             }
         })
         //Next we synchronize the full set that is given by the date limit.
         //We fetch all data for this set.
         //This will also pull in any new messages in subsequent runs.
-        .then([=] {
+        .then([=] (qint64 serverUidNext){
             auto job = [&] {
-                SinkLogCtx(mLogCtx) << "Fetching messages since: " << dateFilter;
                 if (dateFilter.isValid()) {
+                    SinkLogCtx(mLogCtx) << "Fetching messages since: " << dateFilter;
                     return imap->fetchUidsSince(imap->mailboxFromFolder(folder), dateFilter);
                 } else {
+                    SinkLogCtx(mLogCtx) << "Fetching messages.";
                     return imap->fetchUids(imap->mailboxFromFolder(folder));
                 }
             }();
             return job.then([=](const QVector<qint64> &uidsToFetch) {
                 SinkTraceCtx(mLogCtx) << "Received result set " << uidsToFetch;
                 SinkTraceCtx(mLogCtx) << "About to fetch mail" << folder.path();
-                const auto uidNext = syncStore().readValue(folderRemoteId, "uidnext").toLongLong();
+                const auto lastSeenUid = syncStore().readValue(folderRemoteId, "uidnext").toLongLong();
 
-                //Make sure the uids are sorted in reverse order and drop everything below uidNext (so we don't refetch what we already have
+                //Make sure the uids are sorted in reverse order and drop everything below lastSeenUid (so we don't refetch what we already have
                 QVector<qint64> filteredAndSorted = uidsToFetch;
                 qSort(filteredAndSorted.begin(), filteredAndSorted.end(), qGreater<qint64>());
-                auto lowerBound = qLowerBound(filteredAndSorted.begin(), filteredAndSorted.end(), uidNext, qGreater<qint64>());
+                auto lowerBound = qLowerBound(filteredAndSorted.begin(), filteredAndSorted.end(), lastSeenUid, qGreater<qint64>());
                 if (lowerBound != filteredAndSorted.end()) {
                     filteredAndSorted.erase(lowerBound, filteredAndSorted.end());
                 }
@@ -328,9 +348,15 @@ public:
                     }
                 })
                 .then([=] {
-                    SinkLogCtx(mLogCtx) << "UIDMAX: " << *maxUid << folder.path();
+                    SinkLogCtx(mLogCtx) << "Highest found uid: " << *maxUid << folder.path();
                     if (*maxUid > 0) {
                         syncStore().writeValue(folderRemoteId, "uidnext", QByteArray::number(*maxUid));
+                    } else {
+                        if (serverUidNext) {
+                            SinkLogCtx(mLogCtx) << "Storing the server side uidnext: " << serverUidNext << folder.path();
+                            //If we don't receive a mail we should still record the updated uidnext value.
+                            syncStore().writeValue(folderRemoteId, "uidnext", QByteArray::number(serverUidNext - 1));
+                        }
                     }
                     syncStore().writeValue(folderRemoteId, "fullsetLowerbound", QByteArray::number(lowerBoundUid));
                     commit();
@@ -394,12 +420,15 @@ public:
 
     Sink::QueryBase applyMailDefaults(const Sink::QueryBase &query)
     {
-        auto defaultDateFilter = QDate::currentDate().addDays(0 - mDaysToSync);
-        auto queryWithDefaults = query;
-        if (!queryWithDefaults.hasFilter<ApplicationDomain::Mail::Date>()) {
-            queryWithDefaults.filter(ApplicationDomain::Mail::Date::name, QVariant::fromValue(defaultDateFilter));
+        if (mDaysToSync > 0) {
+            auto defaultDateFilter = QDate::currentDate().addDays(0 - mDaysToSync);
+            auto queryWithDefaults = query;
+            if (!queryWithDefaults.hasFilter<ApplicationDomain::Mail::Date>()) {
+                queryWithDefaults.filter(ApplicationDomain::Mail::Date::name, QVariant::fromValue(defaultDateFilter));
+            }
+            return queryWithDefaults;
         }
-        return queryWithDefaults;
+        return query;
     }
 
     QList<Synchronizer::SyncRequest> getSyncRequests(const Sink::QueryBase &query) Q_DECL_OVERRIDE
@@ -470,7 +499,7 @@ public:
     {
         SinkTrace() << "Connecting to:" << mServer << mPort;
         SinkTrace() << "as:" << mUser;
-        return imap->login(mUser, mPassword)
+        return imap->login(mUser, secret())
         .addToContext(imap);
     }
 
@@ -483,6 +512,7 @@ public:
             auto localIds = resolveFilter(folderFilter);
             auto folderRemoteIds = syncStore().resolveLocalIds(ApplicationDomain::getTypeName<ApplicationDomain::Folder>(), localIds);
             for (const auto &r : folderRemoteIds) {
+                Q_ASSERT(!r.isEmpty());
                 folders << Folder{r};
             }
             return KAsync::value(folders);
@@ -509,8 +539,14 @@ public:
                     return {ApplicationDomain::ConnectionError, error.errorMessage};
                 case Imap::SslHandshakeError:
                     return {ApplicationDomain::LoginError, error.errorMessage};
+                case Imap::LoginFailed:
+                    return {ApplicationDomain::LoginError, error.errorMessage};
                 case Imap::HostNotFoundError:
                     return {ApplicationDomain::NoServerError, error.errorMessage};
+                case Imap::ConnectionLost:
+                    return {ApplicationDomain::ConnectionLostError, error.errorMessage};
+                case Imap::MissingCredentialsError:
+                    return {ApplicationDomain::MissingCredentialsError, error.errorMessage};
                 default:
                     return {ApplicationDomain::UnknownError, error.errorMessage};
             }
@@ -520,7 +556,7 @@ public:
 
     KAsync::Job<void> synchronizeWithSource(const Sink::QueryBase &query) Q_DECL_OVERRIDE
     {
-        auto imap = QSharedPointer<ImapServerProxy>::create(mServer, mPort, &mSessionCache);
+        auto imap = QSharedPointer<ImapServerProxy>::create(mServer, mPort, mEncryptionMode, &mSessionCache);
         if (query.type() == ApplicationDomain::getTypeName<ApplicationDomain::Folder>()) {
             return login(imap)
             .then([=] {
@@ -528,8 +564,30 @@ public:
                 return  imap->fetchFolders([folderList](const Folder &folder) {
                     *folderList << folder;
                 })
-                .then([this, folderList]() {
+                .then([=]() {
                     synchronizeFolders(*folderList);
+                    return *folderList;
+                })
+                //The rest is only to check for new messages.
+                .each([=](const Imap::Folder &folder) {
+                    if (!folder.noselect && folder.subscribed) {
+                        return imap->examine(folder)
+                            .then([=](const SelectResult &result) {
+                                const auto folderRemoteId = folderRid(folder);
+                                auto lastSeenUid = syncStore().readValue(folderRemoteId, "uidnext").toLongLong();
+                                SinkTraceCtx(mLogCtx) << "Checking for new messages." << folderRemoteId << " Last seen uid: " << lastSeenUid << " Uidnext: " << result.uidNext;
+                                if (result.uidNext > (lastSeenUid + 1)) {
+                                    const auto folderLocalId = syncStore().resolveRemoteId(ENTITY_TYPE_FOLDER, folderRemoteId);
+                                    emitNotification(Notification::Info, ApplicationDomain::NewContentAvailable, {}, {}, {{folderLocalId}});
+                                }
+                            }).then([=] (const KAsync::Error &error) {
+                                if (error) {
+                                    //Ignore the error because we don't want to fail the synchronization here
+                                    SinkWarningCtx(mLogCtx) << "Examine failed: " << error.errorMessage;
+                                }
+                            });
+                    }
+                    return KAsync::null();
                 });
             })
             .then([=] (const KAsync::Error &error) {
@@ -617,14 +675,39 @@ public:
         }
     }
 
+    static bool validateContent(const QByteArray &data) {
+        if (data.isEmpty()) {
+            SinkError() << "No data available.";
+            return false;
+        }
+        if (data.contains('\0')) {
+            SinkError() << "Data contains NUL, this will fail with IMAP.";
+            return false;
+        }
+        return true;
+    }
+
     KAsync::Job<QByteArray> replay(const ApplicationDomain::Mail &mail, Sink::Operation operation, const QByteArray &oldRemoteId, const QList<QByteArray> &changedProperties) Q_DECL_OVERRIDE
     {
-        auto imap = QSharedPointer<ImapServerProxy>::create(mServer, mPort, &mSessionCache);
-        auto login = imap->login(mUser, mPassword);
+        if (operation != Sink::Operation_Creation) {
+            if(oldRemoteId.isEmpty()) {
+                SinkWarning() << "Tried to replay modification without old remoteId.";
+                // Since we can't recover from the situation we just skip over the revision.
+                // This can for instance happen if creation failed, and we then process a removal or modification.
+                return KAsync::null<QByteArray>();
+            }
+        }
+        auto imap = QSharedPointer<ImapServerProxy>::create(mServer, mPort, mEncryptionMode, &mSessionCache);
+        auto login = imap->login(mUser, secret());
         KAsync::Job<QByteArray> job = KAsync::null<QByteArray>();
         if (operation == Sink::Operation_Creation) {
             const QString mailbox = syncStore().resolveLocalId(ENTITY_TYPE_FOLDER, mail.getFolder());
             const auto content = ensureCRLF(mail.getMimeMessage());
+            if (!validateContent(content)) {
+                SinkError() << "Validation failed during creation replay " << mail.identifier() << "\n  Content:" << content;
+                //We can't recover from this other than deleting the mail, so we skip it.
+                return KAsync::null<QByteArray>();
+            }
             const auto flags = getFlags(mail);
             const QDateTime internalDate = mail.getDate();
             job = login.then(imap->append(mailbox, content, flags, internalDate))
@@ -660,6 +743,11 @@ public:
                 const auto folderId = folderIdFromMailRid(oldRemoteId);
                 const QString oldMailbox = syncStore().resolveLocalId(ENTITY_TYPE_FOLDER, folderId);
                 const auto content = ensureCRLF(mail.getMimeMessage());
+                if (!validateContent(content)) {
+                    SinkError() << "Validation failed during modification replay " << mail.identifier() << "\n  Content:" << content;
+                    //We can't recover from this other than deleting the mail, so we skip it.
+                    return KAsync::null<QByteArray>();
+                }
                 const QDateTime internalDate = mail.getDate();
                 SinkTrace() << "Replacing message. Old mailbox: " << oldMailbox << "New mailbox: " << mailbox << "Flags: " << flags << "Content: " << content;
                 KIMAP2::ImapSet set;
@@ -704,8 +792,8 @@ public:
                 return KAsync::error<QByteArray>("Tried to replay modification without old remoteId.");
             }
         }
-        auto imap = QSharedPointer<ImapServerProxy>::create(mServer, mPort, &mSessionCache);
-        auto login = imap->login(mUser, mPassword);
+        auto imap = QSharedPointer<ImapServerProxy>::create(mServer, mPort, mEncryptionMode, &mSessionCache);
+        auto login = imap->login(mUser, secret());
         if (operation == Sink::Operation_Creation) {
             QString parentFolder;
             if (!folder.getParent().isEmpty()) {
@@ -725,7 +813,7 @@ public:
                     });
             } else { //We try to merge special purpose folders first
                 auto  specialPurposeFolders = QSharedPointer<QHash<QByteArray, QString>>::create();
-                auto mergeJob = imap->login(mUser, mPassword)
+                auto mergeJob = imap->login(mUser, secret())
                     .then(imap->fetchFolders([=](const Imap::Folder &folder) {
                         if (SpecialPurpose::isSpecialPurposeFolderName(folder.name())) {
                             specialPurposeFolders->insert(SpecialPurpose::getSpecialPurposeType(folder.name()), folder.path());
@@ -778,8 +866,8 @@ public:
 public:
     QString mServer;
     int mPort;
+    Imap::EncryptionMode mEncryptionMode = Imap::NoEncryption;
     QString mUser;
-    QString mPassword;
     int mDaysToSync = 0;
     QByteArray mResourceInstanceIdentifier;
     Imap::SessionCache mSessionCache;
@@ -812,6 +900,10 @@ protected:
             const auto folderRemoteId = syncStore->resolveLocalId(ENTITY_TYPE_FOLDER, mail.getFolder());
             const auto mailRemoteId = syncStore->resolveLocalId(ENTITY_TYPE_MAIL, mail.identifier());
             if (mailRemoteId.isEmpty() || folderRemoteId.isEmpty()) {
+                //There is no remote id to find if we expect the message to not exist
+                if (inspectionType == Sink::ResourceControl::Inspection::ExistenceInspectionType && !expectedValue.toBool()) {
+                    return KAsync::null<void>();
+                }
                 SinkWarning() << "Missing remote id for folder or mail. " << mailRemoteId << folderRemoteId;
                 return KAsync::error<void>();
             }
@@ -825,11 +917,11 @@ protected:
             }
             KIMAP2::FetchJob::FetchScope scope;
             scope.mode = KIMAP2::FetchJob::FetchScope::Full;
-            auto imap = QSharedPointer<ImapServerProxy>::create(mServer, mPort);
+            auto imap = QSharedPointer<ImapServerProxy>::create(mServer, mPort, mEncryptionMode);
             auto messageByUid = QSharedPointer<QHash<qint64, Imap::Message>>::create();
             SinkTrace() << "Connecting to:" << mServer << mPort;
             SinkTrace() << "as:" << mUser;
-            auto inspectionJob = imap->login(mUser, mPassword)
+            auto inspectionJob = imap->login(mUser, secret())
                 .then(imap->select(folderRemoteId))
                 .then([](Imap::SelectResult){})
                 .then(imap->fetch(set, scope, [imap, messageByUid](const Imap::Message &message) {
@@ -893,9 +985,9 @@ protected:
                 auto set = KIMAP2::ImapSet::fromImapSequenceSet("1:*");
                 KIMAP2::FetchJob::FetchScope scope;
                 scope.mode = KIMAP2::FetchJob::FetchScope::Headers;
-                auto imap = QSharedPointer<ImapServerProxy>::create(mServer, mPort);
+                auto imap = QSharedPointer<ImapServerProxy>::create(mServer, mPort, mEncryptionMode);
                 auto messageByUid = QSharedPointer<QHash<qint64, Imap::Message>>::create();
-                return imap->login(mUser, mPassword)
+                return imap->login(mUser, secret())
                     .then(imap->select(remoteId))
                     .then(imap->fetch(set, scope, [=](const Imap::Message message) {
                         messageByUid->insert(message.uid, message);
@@ -911,13 +1003,13 @@ protected:
                 auto  folderByPath = QSharedPointer<QSet<QString>>::create();
                 auto  folderByName = QSharedPointer<QSet<QString>>::create();
 
-                auto imap = QSharedPointer<ImapServerProxy>::create(mServer, mPort);
-                auto inspectionJob = imap->login(mUser, mPassword)
+                auto imap = QSharedPointer<ImapServerProxy>::create(mServer, mPort, mEncryptionMode);
+                auto inspectionJob = imap->login(mUser, secret())
                     .then(imap->fetchFolders([=](const Imap::Folder &f) {
                         *folderByPath << f.path();
                         *folderByName << f.name();
                     }))
-                    .then([this, folderByName, folderByPath, folder, remoteId, imap] {
+                    .then([folderByName, folderByPath, folder, remoteId, imap] {
                         if (!folderByName->contains(folder.getName())) {
                             SinkWarning() << "Existing folders are: " << *folderByPath;
                             SinkWarning() << "We're looking for: " << folder.getName();
@@ -936,8 +1028,8 @@ protected:
 public:
     QString mServer;
     int mPort;
+    Imap::EncryptionMode mEncryptionMode = Imap::NoEncryption;
     QString mUser;
-    QString mPassword;
 };
 
 
@@ -948,7 +1040,17 @@ ImapResource::ImapResource(const ResourceContext &resourceContext)
     auto server = config.value("server").toString();
     auto port = config.value("port").toInt();
     auto user = config.value("username").toString();
-    auto password = config.value("password").toString();
+    auto daysToSync = config.value("daysToSync", 14).toInt();
+    auto starttls = config.value("starttls", false).toBool();
+
+    auto encryption = Imap::NoEncryption;
+    if (server.startsWith("imaps")) {
+        encryption = Imap::Tls;
+    }
+    if (starttls) {
+        encryption = Imap::Starttls;
+    }
+
     if (server.startsWith("imap")) {
         server.remove("imap://");
         server.remove("imaps://");
@@ -959,19 +1061,36 @@ ImapResource::ImapResource(const ResourceContext &resourceContext)
         port = list.at(1).toInt();
     }
 
+    //Backwards compatibilty
+    //For kolabnow we assumed that port 143 means starttls
+    if (encryption == Imap::Tls && port == 143) {
+        encryption = Imap::Starttls;
+    }
+
+    if (!QSslSocket::supportsSsl()) {
+        SinkWarning() << "Qt doesn't support ssl. This is likely a distribution/packaging problem.";
+        //On windows this means that the required ssl dll's are missing
+        SinkWarning() << "Ssl Library Build Version Number: " << QSslSocket::sslLibraryBuildVersionString();
+        SinkWarning() << "Ssl Library Runtime Version Number: " << QSslSocket::sslLibraryVersionString();
+    } else {
+        SinkTrace() << "Ssl support available";
+        SinkTrace() << "Ssl Library Build Version Number: " << QSslSocket::sslLibraryBuildVersionString();
+        SinkTrace() << "Ssl Library Runtime Version Number: " << QSslSocket::sslLibraryVersionString();
+    }
+
     auto synchronizer = QSharedPointer<ImapSynchronizer>::create(resourceContext);
     synchronizer->mServer = server;
     synchronizer->mPort = port;
+    synchronizer->mEncryptionMode = encryption;
     synchronizer->mUser = user;
-    synchronizer->mPassword = password;
-    synchronizer->mDaysToSync = 14;
+    synchronizer->mDaysToSync = daysToSync;
     setupSynchronizer(synchronizer);
 
     auto inspector = QSharedPointer<ImapInspector>::create(resourceContext);
     inspector->mServer = server;
     inspector->mPort = port;
+    inspector->mEncryptionMode = encryption;
     inspector->mUser = user;
-    inspector->mPassword = password;
     setupInspector(inspector);
 
     setupPreprocessors(ENTITY_TYPE_MAIL, QVector<Sink::Preprocessor*>() << new SpecialPurposeProcessor << new MailPropertyExtractor);
