@@ -31,6 +31,8 @@
 #include <KIMAP2/ExpungeJob>
 #include <KIMAP2/CapabilitiesJob>
 #include <KIMAP2/SearchJob>
+#include <KIMAP2/GetMetaDataJob>
+#include <KIMAP2/SubscribeJob>
 
 #include <KCoreAddons/KJob>
 
@@ -61,6 +63,34 @@ const char* Imap::Capabilities::Namespace = "NAMESPACE";
 const char* Imap::Capabilities::Uidplus = "UIDPLUS";
 const char* Imap::Capabilities::Condstore = "CONDSTORE";
 
+static int translateImapError(KJob *job)
+{
+    const int error = job->error();
+    const bool isLoginJob = dynamic_cast<KIMAP2::LoginJob*>(job);
+    const bool isSelectJob = dynamic_cast<KIMAP2::SelectJob*>(job);
+    switch (error) {
+        case KIMAP2::LoginJob::ErrorCode::ERR_HOST_NOT_FOUND:
+            return Imap::HostNotFoundError;
+        case KIMAP2::LoginJob::ErrorCode::ERR_COULD_NOT_CONNECT:
+            return Imap::CouldNotConnectError;
+        case KIMAP2::LoginJob::ErrorCode::ERR_SSL_HANDSHAKE_FAILED:
+            return Imap::SslHandshakeError;
+    }
+    //Hack to detect login failures
+    if (isLoginJob) {
+        return Imap::LoginFailed;
+    }
+    //Hack to detect selection errors
+    if (isSelectJob) {
+        return Imap::SelectFailed;
+    }
+    //Hack to detect connection lost
+    if (error == KJob::UserDefinedError) {
+        return Imap::ConnectionLost;
+    }
+    return Imap::UnknownError;
+}
+
 template <typename T>
 static KAsync::Job<T> runJob(KJob *job, const std::function<T(KJob*)> &f)
 {
@@ -68,8 +98,9 @@ static KAsync::Job<T> runJob(KJob *job, const std::function<T(KJob*)> &f)
         QObject::connect(job, &KJob::result, [&future, f](KJob *job) {
             SinkTrace() << "Job done: " << job->metaObject()->className();
             if (job->error()) {
-                SinkWarning() << "Job failed: " << job->errorString() << job->metaObject()->className();
-                future.setError(job->error(), job->errorString());
+                SinkWarning() << "Job failed: " << job->errorString() << job->metaObject()->className() << job->error();
+                auto proxyError = translateImapError(job);
+                future.setError(proxyError, job->errorString());
             } else {
                 future.setValue(f(job));
                 future.setFinished();
@@ -86,8 +117,9 @@ static KAsync::Job<void> runJob(KJob *job)
         QObject::connect(job, &KJob::result, [&future](KJob *job) {
             SinkTrace() << "Job done: " << job->metaObject()->className();
             if (job->error()) {
-                SinkWarning() << "Job failed: " << job->errorString() << job->metaObject()->className();
-                future.setError(job->error(), job->errorString());
+                SinkWarning() << "Job failed: " << job->errorString() << job->metaObject()->className() << job->error();
+                auto proxyError = translateImapError(job);
+                future.setError(proxyError, job->errorString());
             } else {
                 future.setFinished();
             }
@@ -97,14 +129,18 @@ static KAsync::Job<void> runJob(KJob *job)
     });
 }
 
+static int socketTimeout()
+{
+    if (Sink::Test::testModeEnabled()) {
+        return 1;
+    }
+    return 40;
+}
+
 KIMAP2::Session *createNewSession(const QString &serverUrl, int port)
 {
     auto newSession = new KIMAP2::Session(serverUrl, qint16(port));
-    if (Sink::Test::testModeEnabled()) {
-        newSession->setTimeout(1);
-    } else {
-        newSession->setTimeout(40);
-    }
+    newSession->setTimeout(socketTimeout());
     QObject::connect(newSession, &KIMAP2::Session::sslErrors, [=](const QList<QSslError> &errors) {
         SinkLog() << "Received ssl error: " << errors;
         newSession->ignoreErrors(errors);
@@ -112,15 +148,25 @@ KIMAP2::Session *createNewSession(const QString &serverUrl, int port)
     return newSession;
 }
 
-ImapServerProxy::ImapServerProxy(const QString &serverUrl, int port, SessionCache *sessionCache) : mSessionCache(sessionCache), mSession(nullptr)
+ImapServerProxy::ImapServerProxy(const QString &serverUrl, int port, EncryptionMode encryptionMode, SessionCache *sessionCache) : mSessionCache(sessionCache), mSession(nullptr), mEncryptionMode(encryptionMode)
 {
     if (!mSessionCache || mSessionCache->isEmpty()) {
         mSession = createNewSession(serverUrl, port);
     }
 }
 
+QDebug operator<<(QDebug debug, const KIMAP2::MailBoxDescriptor &c)
+{
+    QDebugStateSaver saver(debug);
+    debug.nospace() << c.name;
+    return debug;
+}
+
 KAsync::Job<void> ImapServerProxy::login(const QString &username, const QString &password)
 {
+    if (password.isEmpty()) {
+        return KAsync::error(Imap::MissingCredentialsError);
+    }
     if (mSessionCache) {
         auto session = mSessionCache->getSession();
         if (session.isValid()) {
@@ -131,18 +177,20 @@ KAsync::Job<void> ImapServerProxy::login(const QString &username, const QString
     }
     Q_ASSERT(mSession);
     if (mSession->state() == KIMAP2::Session::Authenticated || mSession->state() == KIMAP2::Session::Selected) {
+        //Prevent the socket from timing out right away, right here (otherwise it just might time out right before we were able to start the job)
+        mSession->setTimeout(socketTimeout());
         SinkLog() << "Reusing existing session.";
         return KAsync::null();
     }
     auto loginJob = new KIMAP2::LoginJob(mSession);
     loginJob->setUserName(username);
     loginJob->setPassword(password);
-    loginJob->setAuthenticationMode(KIMAP2::LoginJob::Plain);
-    if (mSession->port() == 143) {
+    if (mEncryptionMode == Starttls) {
         loginJob->setEncryptionMode(QSsl::TlsV1_0OrLater, true);
-    } else {
+    } else if (mEncryptionMode == Tls) {
         loginJob->setEncryptionMode(QSsl::AnyProtocol, false);
     }
+    loginJob->setAuthenticationMode(KIMAP2::LoginJob::Plain);
 
     auto capabilitiesJob = new KIMAP2::CapabilitiesJob(mSession);
     QObject::connect(capabilitiesJob, &KIMAP2::CapabilitiesJob::capabilitiesReceived, &mGuard, [this](const QStringList &capabilities) {
@@ -166,20 +214,6 @@ KAsync::Job<void> ImapServerProxy::login(const QString &username, const QString
         // SinkTrace() << "Found personal namespaces: " << mNamespaces.personal;
         // SinkTrace() << "Found shared namespaces: " << mNamespaces.shared;
         // SinkTrace() << "Found user namespaces: " << mNamespaces.user;
-    }).then([=] (const KAsync::Error &error) {
-        if (error) {
-            switch (error.errorCode) {
-            case KIMAP2::LoginJob::ErrorCode::ERR_HOST_NOT_FOUND:
-                return KAsync::error(HostNotFoundError, "Host not found: " + error.errorMessage);
-            case KIMAP2::LoginJob::ErrorCode::ERR_COULD_NOT_CONNECT:
-                return KAsync::error(CouldNotConnectError, "Failed to connect: " + error.errorMessage);
-            case KIMAP2::LoginJob::ErrorCode::ERR_SSL_HANDSHAKE_FAILED:
-                return KAsync::error(SslHandshakeError, "Ssl handshake failed: " + error.errorMessage);
-            default:
-                return KAsync::error(error);
-            }
-        }
-        return KAsync::null();
     });
 }
 
@@ -217,6 +251,29 @@ KAsync::Job<SelectResult> ImapServerProxy::select(const QString &mailbox)
     });
 }
 
+KAsync::Job<SelectResult> ImapServerProxy::select(const Folder &folder)
+{
+    return select(mailboxFromFolder(folder));
+}
+
+KAsync::Job<SelectResult> ImapServerProxy::examine(const QString &mailbox)
+{
+    auto select = new KIMAP2::SelectJob(mSession);
+    select->setOpenReadOnly(true);
+    select->setMailBox(mailbox);
+    select->setCondstoreEnabled(mCapabilities.contains(Capabilities::Condstore));
+    return runJob<SelectResult>(select, [select](KJob* job) -> SelectResult {
+        return {select->uidValidity(), select->nextUid(), select->highestModSequence()};
+    }).onError([=] (const KAsync::Error &error) {
+        SinkWarning() << "Examine failed: " << mailbox;
+    });
+}
+
+KAsync::Job<SelectResult> ImapServerProxy::examine(const Folder &folder)
+{
+    return examine(mailboxFromFolder(folder));
+}
+
 KAsync::Job<qint64> ImapServerProxy::append(const QString &mailbox, const QByteArray &content, const QList<QByteArray> &flags, const QDateTime &internalDate)
 {
     auto append = new KIMAP2::AppendJob(mSession);
@@ -271,6 +328,13 @@ KAsync::Job<void> ImapServerProxy::create(const QString &mailbox)
     return runJob(create);
 }
 
+KAsync::Job<void> ImapServerProxy::subscribe(const QString &mailbox)
+{
+    auto job = new KIMAP2::SubscribeJob(mSession);
+    job->setMailBox(mailbox);
+    return runJob(job);
+}
+
 KAsync::Job<void> ImapServerProxy::rename(const QString &mailbox, const QString &newMailbox)
 {
     auto rename = new KIMAP2::RenameJob(mSession);
@@ -373,7 +437,6 @@ KAsync::Job<QVector<qint64>> ImapServerProxy::fetchHeaders(const QString &mailbo
 
 KAsync::Job<QVector<qint64>> ImapServerProxy::fetchUids(const QString &mailbox)
 {
-    auto term = KIMAP2::Term(KIMAP2::Term::Uid, KIMAP2::ImapSet(1, 0));
     auto notDeleted = KIMAP2::Term(KIMAP2::Term::Deleted);
     notDeleted.setNegated(true);
     return select(mailbox).then<QVector<qint64>>(search(notDeleted));
@@ -476,14 +539,33 @@ static void reportFolder(const Folder &f, QSharedPointer<QSet<QString>> reported
     }
 }
 
+KAsync::Job<void> ImapServerProxy::getMetaData(std::function<void(const QHash<QString, QMap<QByteArray, QByteArray> > &metadata)> callback)
+{
+    if (!mCapabilities.contains("METADATA")) {
+        return KAsync::null();
+    }
+    KIMAP2::GetMetaDataJob *meta = new KIMAP2::GetMetaDataJob(mSession);
+    meta->setMailBox(QLatin1String("*"));
+    meta->setServerCapability( KIMAP2::MetaDataJobBase::Metadata );
+    meta->setDepth(KIMAP2::GetMetaDataJob::AllLevels);
+    meta->addRequestedEntry("/shared/vendor/kolab/folder-type");
+    meta->addRequestedEntry("/private/vendor/kolab/folder-type");
+    return runJob(meta).then<void>([callback, meta] () {
+        callback(meta->allMetaDataForMailboxes());
+    });
+}
+
 KAsync::Job<void> ImapServerProxy::fetchFolders(std::function<void(const Folder &)> callback)
 {
     SinkTrace() << "Fetching folders";
     auto subscribedList = QSharedPointer<QSet<QString>>::create() ;
     auto reportedList = QSharedPointer<QSet<QString>>::create() ;
-    return list(KIMAP2::ListJob::NoOption, [=](const KIMAP2::MailBoxDescriptor &mailbox, const QList<QByteArray> &){
+    auto metaData = QSharedPointer<QHash<QString, QMap<QByteArray, QByteArray>>>::create() ;
+    return getMetaData([=] (const QHash<QString, QMap<QByteArray, QByteArray>> &m) {
+        *metaData = m;
+    }).then(list(KIMAP2::ListJob::NoOption, [=](const KIMAP2::MailBoxDescriptor &mailbox, const QList<QByteArray> &){
         *subscribedList << mailbox.name;
-    }).then(list(KIMAP2::ListJob::IncludeUnsubscribed, [=](const KIMAP2::MailBoxDescriptor &mailbox, const QList<QByteArray> &flags) {
+    })).then(list(KIMAP2::ListJob::IncludeUnsubscribed, [=](const KIMAP2::MailBoxDescriptor &mailbox, const QList<QByteArray> &flags) {
         bool noselect = caseInsensitiveContains(FolderFlags::Noselect, flags);
         bool subscribed = subscribedList->contains(mailbox.name);
         if (isGmail()) {
@@ -499,6 +581,17 @@ KAsync::Job<void> ImapServerProxy::fetchFolders(std::function<void(const Folder
             }
         }
         SinkLog() << "Found mailbox: " << mailbox.name << flags << FolderFlags::Noselect << noselect  << " sub: " << subscribed;
+        //Ignore all non-mail folders
+        if (metaData->contains(mailbox.name)) {
+            auto m = metaData->value(mailbox.name);
+            auto sharedType = m.value("/shared/vendor/kolab/folder-type");
+            auto privateType = m.value("/private/vendor/kolab/folder-type");
+            auto type = !privateType.isEmpty() ? privateType : sharedType;
+            if (!type.isEmpty() && !type.contains("mail")) {
+                SinkLog() << "Skipping due to folder type: " << type;
+                return;
+            }
+        }
         auto ns = getNamespace(mailbox.name);
         auto folder = Folder{mailbox.name, ns, mailbox.separator, noselect, subscribed, flags};
 
@@ -521,7 +614,7 @@ QString ImapServerProxy::mailboxFromFolder(const Folder &folder) const
 KAsync::Job<SelectResult> ImapServerProxy::fetchFlags(const Folder &folder, const KIMAP2::ImapSet &set, qint64 changedsince, std::function<void(const Message &)> callback)
 {
     SinkTrace() << "Fetching flags " << folder.path();
-    return select(mailboxFromFolder(folder)).then<SelectResult, SelectResult>([=](const SelectResult &selectResult) -> KAsync::Job<SelectResult> {
+    return select(folder).then<SelectResult, SelectResult>([=](const SelectResult &selectResult) -> KAsync::Job<SelectResult> {
         SinkTrace() << "Modeseq " << folder.path() << selectResult.highestModSequence << changedsince;
 
         if (selectResult.highestModSequence == static_cast<quint64>(changedsince)) {
@@ -545,7 +638,7 @@ KAsync::Job<void> ImapServerProxy::fetchMessages(const Folder &folder, qint64 ui
 {
     auto time = QSharedPointer<QTime>::create();
     time->start();
-    return select(mailboxFromFolder(folder)).then<void, SelectResult>([this, callback, folder, time, progress, uidNext](const SelectResult &selectResult) -> KAsync::Job<void> {
+    return select(folder).then<void, SelectResult>([this, callback, folder, time, progress, uidNext](const SelectResult &selectResult) -> KAsync::Job<void> {
         SinkTrace() << "UIDNEXT " << folder.path() << selectResult.uidNext << uidNext;
         if (selectResult.uidNext == (uidNext + 1)) {
             SinkTrace()<< folder.path() << "Uidnext didn't change, nothing to do.";
@@ -568,7 +661,7 @@ KAsync::Job<void> ImapServerProxy::fetchMessages(const Folder &folder, const QVe
 {
     auto time = QSharedPointer<QTime>::create();
     time->start();
-    return select(mailboxFromFolder(folder)).then<void, SelectResult>([this, callback, folder, time, progress, uidsToFetch, headersOnly](const SelectResult &selectResult) -> KAsync::Job<void> {
+    return select(folder).then<void, SelectResult>([this, callback, folder, time, progress, uidsToFetch, headersOnly](const SelectResult &selectResult) -> KAsync::Job<void> {
 
         SinkTrace() << "Fetching messages" << folder.path();
         SinkTrace() << "  Total: " << uidsToFetch.size();
