@@ -22,6 +22,7 @@
 
 #include "common/store.h"
 #include "common/log.h"
+#include "common/propertyparser.h"
 
 #include "utils.h"
 
@@ -67,7 +68,7 @@ QList<QByteArray> requestedProperties(const QString &type)
     } else if (type == getTypeName<Addressbook>()) {
         return QList<QByteArray>() << Addressbook::Name::name << Addressbook::Parent::name;
     } else if (type == getTypeName<SinkResource>()) {
-        return QList<QByteArray>() << SinkResource::ResourceType::name << SinkResource::Account::name << SinkResource::Capabilities::name;
+        return QList<QByteArray>() << SinkResource::ResourceType::name << SinkResource::Account::name;
     } else if (type == getTypeName<SinkAccount>()) {
         return QList<QByteArray>() << SinkAccount::AccountType::name << SinkAccount::Name::name;
     } else if (type == getTypeName<Identity>()) {
@@ -163,7 +164,7 @@ bool applyFilter(Sink::Query &query, const QStringList &args_)
         if (resource.contains('/')) {
             //The resource isn't an id but a path
             auto list = resource.split('/');
-            const auto resourceId = list.takeFirst();
+            const auto resourceId = parseUid(list.takeFirst());
             query.resourceFilter(resourceId);
             if (type == Sink::ApplicationDomain::getTypeName<Sink::ApplicationDomain::Mail>() && !list.isEmpty()) {
                 auto value = list.takeFirst();
@@ -187,7 +188,7 @@ bool applyFilter(Sink::Query &query, const QStringList &args_)
                 }
             }
         } else {
-            query.resourceFilter(resource);
+            query.resourceFilter(parseUid(resource));
         }
     }
     return true;
@@ -195,7 +196,39 @@ bool applyFilter(Sink::Query &query, const QStringList &args_)
 
 bool applyFilter(Sink::Query &query, const SyntaxTree::Options &options)
 {
-    return applyFilter(query, options.positionalArguments);
+    bool ret = applyFilter(query, options.positionalArguments);
+    if (options.options.contains("resource")) {
+        for (const auto &f : options.options.value("resource")) {
+            query.resourceFilter(parseUid(f.toLatin1()));
+        }
+    }
+    if (options.options.contains("filter")) {
+        for (const auto &f : options.options.value("filter")) {
+            auto filter = f.split("=");
+            const auto property = filter.value(0).toLatin1();
+            const auto value = filter.value(1);
+            query.filter(property, Sink::PropertyParser::parse(query.type(), property, QString::fromUtf8(parseUid(value.toUtf8()))));
+        }
+    }
+    if (options.options.contains("fulltext")) {
+        for (const auto &f : options.options.value("fulltext")) {
+            query.filter({}, Sink::QueryBase::Comparator(f, Sink::QueryBase::Comparator::Fulltext));
+        }
+    }
+    if (options.options.contains("id")) {
+        for (const auto &f : options.options.value("id")) {
+            query.filter(parseUid(f.toUtf8()));
+        }
+    }
+    return ret;
+}
+
+QByteArray parseUid(const QByteArray &uid)
+{
+    if (uid.size() == 36 && uid.contains('-') && !uid.startsWith('{')) {
+        return '{' + uid + '}';
+    }
+    return uid;
 }
 
 }
