@@ -13,7 +13,6 @@
 
 template <typename T>
 struct Result {
-    QSharedPointer<T> parent;
     bool fetchedAll;
 };
 
@@ -73,37 +72,28 @@ public:
         // We have to do it this way, otherwise we're not setting the fetcher right
         auto emitter = resultProvider->emitter();
 
-        resultProvider->setFetcher([query, resultProvider, this, ctx](const typename T::Ptr &parent) {
+        resultProvider->setFetcher([query, resultProvider, this, ctx]() {
             async::run<Result<T>>([=] {
-                if (parent) {
-                    SinkTraceCtx(ctx) << "Running the fetcher " << parent->identifier();
-                } else {
-                    SinkTraceCtx(ctx) << "Running the fetcher.";
-                }
+                SinkTraceCtx(ctx) << "Running the fetcher.";
                 SinkTraceCtx(ctx) << "-------------------------.";
                 int count = 0;
                 for (int i = offset; i < results.size(); i++) {
                     const auto res = results.at(i);
                     count++;
-                    // SinkTraceCtx(ctx) << "Parent filter " << query.getFilter("parent").value.toByteArray() << res->identifier() << res->getProperty("parent").toByteArray();
-                    auto parentProperty = res->getProperty("parent").toByteArray();
-                    if ((!parent && parentProperty.isEmpty()) || (parent && parentProperty == parent->identifier()) || query.parentProperty().isEmpty()) {
-                        // SinkTraceCtx(ctx) << "Found a hit" << res->identifier();
-                        resultProvider->add(res);
-                    }
+                    resultProvider->add(res);
                     if (query.limit()) {
                         if (count >= query.limit()) {
                             SinkTraceCtx(ctx) << "Aborting early after " << count << "results.";
                             offset = i + 1;
                             bool fetchedAll = (i + 1 >= results.size());
-                            return Result<T>{parent, fetchedAll};
+                            return Result<T>{fetchedAll};
                         }
                     }
                 }
-                return Result<T>{parent, true};
+                return Result<T>{true};
             }, runAsync)
             .then([=] (const Result<T> &r) {
-                resultProvider->initialResultSetComplete(r.parent, r.fetchedAll);
+                resultProvider->initialResultSetComplete(r.fetchedAll);
             })
             .exec();
         });
@@ -185,30 +175,10 @@ private slots:
         QTRY_COMPARE(model->rowCount(), 1);
     }
 
-    void testModelNested()
-    {
-        auto facade = setupFacade<Sink::ApplicationDomain::Folder>("dummyresource.instance1");
-        auto folder = QSharedPointer<Sink::ApplicationDomain::Folder>::create("resource", "id", 0, QSharedPointer<Sink::ApplicationDomain::MemoryBufferAdaptor>::create());
-        auto subfolder = QSharedPointer<Sink::ApplicationDomain::Folder>::create("resource", "subId", 0, QSharedPointer<Sink::ApplicationDomain::MemoryBufferAdaptor>::create());
-        subfolder->setParent("id");
-        facade->results << folder << subfolder;
-
-        // Test
-        Sink::Query query;
-        query.resourceFilter("dummyresource.instance1");
-        query.requestTree("parent");
-
-        auto model = Sink::Store::loadModel<Sink::ApplicationDomain::Folder>(query);
-        QTRY_VERIFY(model->data(QModelIndex(), Sink::Store::ChildrenFetchedRole).toBool());
-        QCOMPARE(model->rowCount(), 1);
-        model->fetchMore(model->index(0, 0));
-        QTRY_VERIFY(model->data(model->index(0, 0), Sink::Store::ChildrenFetchedRole).toBool());
-        QCOMPARE(model->rowCount(model->index(0, 0)), 1);
-    }
-
     void testModelSignals()
     {
         auto facade = setupFacade<Sink::ApplicationDomain::Folder>("dummyresource.instance1");
+        facade->runAsync = true;
         auto folder = QSharedPointer<Sink::ApplicationDomain::Folder>::create("resource", "id", 0, QSharedPointer<Sink::ApplicationDomain::MemoryBufferAdaptor>::create());
         auto subfolder = QSharedPointer<Sink::ApplicationDomain::Folder>::create("resource", "subId", 0, QSharedPointer<Sink::ApplicationDomain::MemoryBufferAdaptor>::create());
         subfolder->setParent("id");
@@ -222,8 +192,64 @@ private slots:
         auto model = Sink::Store::loadModel<Sink::ApplicationDomain::Folder>(query);
         QSignalSpy spy(model.data(), SIGNAL(rowsInserted(const QModelIndex &, int, int)));
         QVERIFY(spy.isValid());
-        model->fetchMore(model->index(0, 0));
-        QTRY_VERIFY(spy.count() >= 1);
+        QTRY_VERIFY(spy.count() == 2);
+    }
+
+    void testModelNested()
+    {
+        auto facade = setupFacade<Sink::ApplicationDomain::Folder>("dummyresource.instance1");
+        facade->runAsync = true;
+        auto folder = QSharedPointer<Sink::ApplicationDomain::Folder>::create("resource", "id", 0, QSharedPointer<Sink::ApplicationDomain::MemoryBufferAdaptor>::create());
+        auto subfolder = QSharedPointer<Sink::ApplicationDomain::Folder>::create("resource", "subId", 0, QSharedPointer<Sink::ApplicationDomain::MemoryBufferAdaptor>::create());
+        subfolder->setParent("id");
+        facade->results << folder << subfolder;
+
+        // Test
+        Sink::Query query;
+        query.resourceFilter("dummyresource.instance1");
+        query.requestTree("parent");
+
+        auto model = Sink::Store::loadModel<Sink::ApplicationDomain::Folder>(query);
+        QObject::connect(model.data(), &QAbstractItemModel::rowsInserted, [&] (const QModelIndex &parent, int first, int last) {
+            for (int row = first; row <= last; row++) {
+                auto index = model->index(row, 0, parent);
+                QVERIFY(index.isValid());
+                QVERIFY(index.data(Sink::Store::DomainObjectRole).value<Sink::ApplicationDomain::Folder::Ptr>());
+            }
+        });
+        QTRY_COMPARE(model->rowCount(), 1);
+        QTRY_COMPARE(model->rowCount(model->index(0, 0)), 1);
+    }
+
+    void testModelNestedReverse()
+    {
+        auto facade = setupFacade<Sink::ApplicationDomain::Folder>("dummyresource.instance1");
+        facade->runAsync = true;
+        auto folder = QSharedPointer<Sink::ApplicationDomain::Folder>::create("resource", "id", 0, QSharedPointer<Sink::ApplicationDomain::MemoryBufferAdaptor>::create());
+        auto subfolder = QSharedPointer<Sink::ApplicationDomain::Folder>::create("resource", "subId", 0, QSharedPointer<Sink::ApplicationDomain::MemoryBufferAdaptor>::create());
+        subfolder->setParent(folder->identifier());
+
+        auto subsubfolder = QSharedPointer<Sink::ApplicationDomain::Folder>::create("resource", "subsubId", 0, QSharedPointer<Sink::ApplicationDomain::MemoryBufferAdaptor>::create());
+        subsubfolder->setParent(subfolder->identifier());
+
+        facade->results << subsubfolder << subfolder << folder;
+
+        // Test
+        Sink::Query query;
+        query.resourceFilter("dummyresource.instance1");
+        query.requestTree("parent");
+
+        auto model = Sink::Store::loadModel<Sink::ApplicationDomain::Folder>(query);
+        QObject::connect(model.data(), &QAbstractItemModel::rowsInserted, [&] (const QModelIndex &parent, int first, int last) {
+            for (int row = first; row <= last; row++) {
+                auto index = model->index(row, 0, parent);
+                QVERIFY(index.isValid());
+                QVERIFY(index.data(Sink::Store::DomainObjectRole).value<Sink::ApplicationDomain::Folder::Ptr>());
+            }
+        });
+        QTRY_COMPARE(model->rowCount(), 1);
+        QVERIFY(model->index(0, 0).data(Sink::Store::DomainObjectRole).value<Sink::ApplicationDomain::Folder::Ptr>());
+        QTRY_COMPARE(model->rowCount(model->index(0, 0)), 1);
     }
 
     void testModelNestedLive()
@@ -360,6 +386,8 @@ private slots:
         auto event = Sink::ApplicationDomain::Event::createEntity<Sink::ApplicationDomain::Event>("dummyresource.instance1");
         Sink::Store::create(event).exec().waitForFinished();
         QCOMPARE(facade->creations.size(), 1);
+        //Modify something so the mdofication won't be dropped
+        event.setExtractedSummary("foobar");
         Sink::Store::modify(event).exec().waitForFinished();
         QCOMPARE(facade->modifications.size(), 1);
         Sink::Store::remove(event).exec().waitForFinished();
@@ -376,7 +404,7 @@ private slots:
         query.resourceFilter("dummyresource.instance1");
 
         auto event = Sink::ApplicationDomain::Event::createEntity<Sink::ApplicationDomain::Event>("dummyresource.instance1");
-        event.setUid("modifiedUid");
+        event.setExtractedUid("modifiedUid");
         Sink::Store::modify(query, event).exec().waitForFinished();
         QCOMPARE(facade->modifications.size(), 2);
         for (const auto &m : facade->modifications) {
@@ -384,6 +412,26 @@ private slots:
         }
     }
 
+    void testAggregateModify()
+    {
+        auto facade = setupFacade<Sink::ApplicationDomain::Event>("dummyresource.instance1");
+        facade->results << QSharedPointer<Sink::ApplicationDomain::Event>::create("dummyresource.instance1", "id1", 0, QSharedPointer<Sink::ApplicationDomain::MemoryBufferAdaptor>::create());
+        facade->results << QSharedPointer<Sink::ApplicationDomain::Event>::create("dummyresource.instance1", "id2", 0, QSharedPointer<Sink::ApplicationDomain::MemoryBufferAdaptor>::create());
+
+        Sink::ApplicationDomain::Event modification("dummyresource.instance1", "id1", 0, QSharedPointer<Sink::ApplicationDomain::MemoryBufferAdaptor>::create());
+        modification.aggregatedIds() << "id1" << "id2";
+        modification.setExtractedUid("modifiedUid2");
+
+        Sink::Store::modify(modification).exec().waitForFinished();
+        QCOMPARE(facade->modifications.size(), 2);
+        for (const auto &m : facade->modifications) {
+            QCOMPARE(m.getUid(), {"modifiedUid2"});
+        }
+
+        Sink::Store::remove(modification).exec().waitForFinished();
+        QCOMPARE(facade->removals.size(), 2);
+    }
+
     void testModelStress()
     {
         auto facade = setupFacade<Sink::ApplicationDomain::Folder>("dummyresource.instance1");
