@@ -14,6 +14,7 @@
 #include "test.h"
 #include "testutils.h"
 #include "adaptorfactoryregistry.h"
+#include "notifier.h"
 
 using namespace Sink;
 using namespace Sink::ApplicationDomain;
@@ -243,7 +244,7 @@ private slots:
         event.setProperty("uid", "testuid");
         QCOMPARE(event.getProperty("uid").toByteArray(), QByteArray("testuid"));
         event.setProperty("summary", "summaryValue");
-        Sink::Store::create<Event>(event).exec().waitForFinished();
+        VERIFYEXEC(Sink::Store::create<Event>(event));
 
         // Test create
         Event event2;
