@@ -33,6 +33,16 @@
 using namespace Sink;
 using namespace Sink::ApplicationDomain;
 
+static QByteArray newMessage(const QString &subject)
+{
+    auto msg = KMime::Message::Ptr::create();
+    msg->subject(true)->fromUnicodeString(subject, "utf8");
+    msg->date(true)->setDateTime(QDateTime::currentDateTimeUtc());
+    msg->assemble();
+    return msg->encodedContent(true);
+}
+
+
 void MailSyncTest::initTestCase()
 {
     Test::initTest();
@@ -261,6 +271,8 @@ void MailSyncTest::testListRemovedSubFolder()
 
 void MailSyncTest::testListMails()
 {
+    createMessage(QStringList() << "test", newMessage("This is a Subject."));
+
     Sink::Query query;
     query.resourceFilter(mResourceInstanceIdentifier);
     query.request<Mail::Subject>().request<Mail::MimeMessage>().request<Mail::Folder>().request<Mail::Date>();
@@ -272,7 +284,7 @@ void MailSyncTest::testListMails()
     auto job = Store::fetchAll<Mail>(query).then([](const QList<Mail::Ptr> &mails) {
         ASYNCCOMPARE(mails.size(), 1);
         auto mail = mails.first();
-        ASYNCVERIFY(mail->getSubject().startsWith(QString("[Nepomuk] Jenkins build is still unstable")));
+        ASYNCVERIFY(mail->getSubject().startsWith(QString("This is a Subject.")));
         const auto data = mail->getMimeMessage();
         ASYNCVERIFY(!data.isEmpty());
 
@@ -306,7 +318,7 @@ void MailSyncTest::testResyncMails()
         ASYNCCOMPARE(mails.size(), 1);
         auto mail = mails.first();
         ASYNCVERIFY(!mail->getSubject().isEmpty());
-        ASYNCVERIFY(!mail->getMimeMessagePath().isEmpty());
+        ASYNCVERIFY(!mail->getMimeMessage().isEmpty());
         return KAsync::null();
     });
     VERIFYEXEC(job);
@@ -322,10 +334,7 @@ void MailSyncTest::testFetchNewRemovedMessages()
     VERIFYEXEC(Store::synchronize(query));
     VERIFYEXEC(ResourceControl::flushMessageQueue(QByteArrayList() << mResourceInstanceIdentifier));
 
-    auto msg = KMime::Message::Ptr::create();
-    msg->subject(true)->fromUnicodeString("Foobar", "utf8");
-    msg->assemble();
-    auto messageIdentifier = createMessage(QStringList() << "test", msg->encodedContent(true));
+    auto messageIdentifier = createMessage(QStringList() << "test", newMessage("Foobar"));
 
     VERIFYEXEC(Store::synchronize(query));
     VERIFYEXEC(ResourceControl::flushMessageQueue(QByteArrayList() << mResourceInstanceIdentifier));
@@ -354,42 +363,29 @@ void MailSyncTest::testFetchNewRemovedMessages()
 
 void MailSyncTest::testFlagChange()
 {
+    Sink::Query syncScope;
+    syncScope.resourceFilter(mResourceInstanceIdentifier);
+
     Sink::Query query;
     query.resourceFilter(mResourceInstanceIdentifier);
     query.filter<Mail::Important>(true);
+    query.filter<Mail::Folder>(Sink::Query{}.filter<Folder::Name>("test"));
     query.request<Mail::Subject>().request<Mail::Important>();
 
-    auto msg = KMime::Message::Ptr::create();
-    msg->subject(true)->fromUnicodeString("Foobar", "utf8");
-    msg->assemble();
-    auto messageIdentifier = createMessage(QStringList() << "test", msg->encodedContent(true));
+    auto messageIdentifier = createMessage(QStringList() << "test", newMessage("Foobar"));
 
-    VERIFYEXEC(Store::synchronize(query));
-    VERIFYEXEC(ResourceControl::flushMessageQueue(QByteArrayList() << mResourceInstanceIdentifier));
+    VERIFYEXEC(Store::synchronize(syncScope));
+    VERIFYEXEC(ResourceControl::flushMessageQueue(mResourceInstanceIdentifier));
 
-    {
-        auto job = Store::fetchAll<Mail>(query).then([](const QList<Mail::Ptr> &mails) {
-            ASYNCCOMPARE(mails.size(), 0);
-            return KAsync::null();
-        });
-        VERIFYEXEC(job);
-    }
+    QCOMPARE(Store::read<Mail>(query).size(), 0);
 
     markAsImportant(QStringList() << "test", messageIdentifier);
 
     // Ensure all local data is processed
-    VERIFYEXEC(Store::synchronize(query));
-    VERIFYEXEC(ResourceControl::flushMessageQueue(QByteArrayList() << mResourceInstanceIdentifier));
-
-    {
-        auto job = Store::fetchAll<Mail>(query).then([](const QList<Mail::Ptr> &mails) {
-            ASYNCCOMPARE(mails.size(), 1);
-            ASYNCVERIFY(mails.first()->getImportant());
-            return KAsync::null();
-        });
-        VERIFYEXEC(job);
-    }
+    VERIFYEXEC(Store::synchronize(syncScope));
+    VERIFYEXEC(ResourceControl::flushMessageQueue(mResourceInstanceIdentifier));
 
+    QCOMPARE(Store::read<Mail>(query).size(), 1);
 }
 
 void MailSyncTest::testSyncSingleFolder()
@@ -414,8 +410,6 @@ void MailSyncTest::testSyncSingleFolder()
     // Ensure all local data is processed
     VERIFYEXEC(Store::synchronize(syncScope));
     VERIFYEXEC(ResourceControl::flushMessageQueue(mResourceInstanceIdentifier));
-
-
 }
 
 void MailSyncTest::testSyncSingleMail()
@@ -468,14 +462,16 @@ void MailSyncTest::testFailingSync()
 
     bool errorReceived = false;
 
-    //We currently don't get a proper error notification except for the status change
+    //Wait for the error notifiction
     auto notifier = QSharedPointer<Sink::Notifier>::create(resource.identifier());
     notifier->registerHandler([&](const Notification &notification) {
-        SinkTrace() << "Received notification " << notification.type << notification.id;
-        if (notification.code == ApplicationDomain::ErrorStatus) {
+        SinkTrace() << "Received notification " << notification;
+        //Maildir detects misconfiguration, imap fails to connect
+        if (notification.type == Sink::Notification::Error &&
+                (notification.code == ApplicationDomain::ConnectionError ||
+                 notification.code == ApplicationDomain::ConfigurationError)) {
             errorReceived = true;
-            SinkWarning() << "Sync return an error";
-        } 
+        }
     });
 
     VERIFYEXEC(Store::synchronize(query));
@@ -483,4 +479,49 @@ void MailSyncTest::testFailingSync()
     QTRY_VERIFY(errorReceived);
 }
 
-#include "mailsynctest.moc"
+void MailSyncTest::testSyncUidvalidity()
+{
+    createFolder({"uidvalidity"});
+    createMessage({"uidvalidity"}, newMessage("old"));
+
+    VERIFYEXEC(Store::synchronize(SyncScope{ApplicationDomain::getTypeName<Folder>()}.resourceFilter(mResourceInstanceIdentifier)));
+    VERIFYEXEC(ResourceControl::flushMessageQueue(mResourceInstanceIdentifier));
+
+    auto folder = Store::readOne<Folder>(Query{}.resourceFilter(mResourceInstanceIdentifier).filter<Folder::Name>("uidvalidity"));
+
+    auto folderSyncScope = SyncScope{ApplicationDomain::getTypeName<Mail>()};
+    folderSyncScope.resourceFilter(mResourceInstanceIdentifier);
+    folderSyncScope.filter<Mail::Folder>(QVariant::fromValue(folder.identifier()));
+    VERIFYEXEC(Store::synchronize(folderSyncScope));
+    VERIFYEXEC(ResourceControl::flushMessageQueue(mResourceInstanceIdentifier));
+
+
+    {
+        Sink::Query query;
+        query.resourceFilter(mResourceInstanceIdentifier);
+        query.request<Mail::Subject>().request<Mail::MimeMessage>().request<Mail::Folder>().request<Mail::Date>();
+        query.filter<Mail::Folder>(folder);
+        auto mails = Store::read<Mail>(query);
+        QCOMPARE(mails.size(), 1);
+    }
+
+    resetTestEnvironment();
+
+    createFolder({"uidvalidity"});
+    createMessage({"uidvalidity"}, newMessage("new"));
+
+    // Ensure all local data is processed
+    VERIFYEXEC(Store::synchronize(folderSyncScope));
+    VERIFYEXEC(ResourceControl::flushMessageQueue(mResourceInstanceIdentifier));
+
+    //Now we should have one message
+    auto folder2 = Store::readOne<Folder>(Query{}.resourceFilter(mResourceInstanceIdentifier).filter<Folder::Name>("uidvalidity"));
+    Sink::Query query;
+    query.resourceFilter(mResourceInstanceIdentifier);
+    query.request<Mail::Subject>().request<Mail::MimeMessage>().request<Mail::Folder>().request<Mail::Date>();
+    query.filter<Mail::Folder>(folder2);
+    auto mails = Store::read<Mail>(query);
+    QCOMPARE(mails.size(), 1);
+    QCOMPARE(mails.first().getSubject(), {"new"});
+}
+
